# boxcount_info.py

# By Nathan C. Hearn
#    June 10, 2008
#
# Reader for boxcount output


class BoxcountInfo :

    def __init__(self) :

        self.__resolution = int(0)
        self.__flame_speed = float(0.0)

        self.__file_index = int(0)
        self.__sim_time = float(0.0)

        self.__box_width = float(0.0)
        self.__threshold_value = float(0.0)

        self.__active_boxes = int(0)
        self.__boxes_above = int(0)
        self.__boxes_below = int(0)
        self.__transition_boxes = int(0)


    def reset(self, resolution, flame_speed, file_index, sim_time, box_width,
              threshold_value, active_boxes, boxes_above, boxes_below,
              transition_boxes) :

        self.__resolution = resolution
        self.__flame_speed = flame_speed

        self.__file_index = file_index
        self.__sim_time = sim_time

        self.__box_width = box_width
        self.__threshold_value = threshold_value

        self.__active_boxes = active_boxes
        self.__boxes_above = boxes_above
        self.__boxes_below = boxes_below
        self.__transition_boxes = transition_boxes


    def get_resolution(self) :
        return self.__resolution

    def get_flame_speed(self) :
        return self.__flame_speed

    def get_file_index(self) :
        return self.__file_index

    def get_sim_time(self) :
        return self.__sim_time

    def get_box_width(self) :
        return self.__box_width

    def get_threshold_value(self) :
        return self.__threshold_value

    def get_active_boxes(self) :
        return self.__active_boxes

    def get_boxes_above(self) :
        return self.__boxes_above

    def get_boxes_below(self) :
        return self.__boxes_below

    def get_transition_boxes(self) :
        return self.__transition_boxes


def read_boxcount_file(filename) :

    ## Get information from filename

    ## THIS COULD BE DONE SOOOOO MUCH BETTER ....

    filename_parts = filename.split("_")

    num_parts = len(filename_parts)

    part_index = int(0)

    ## The resolution string is the first element that is only digits

    found_resolution = False

    while ((not found_resolution) and (part_index < num_parts)) :
        if (filename_parts[part_index].isdigit()) :
            found_resolution = True
        else :
            part_index += 1

    if (not found_resolution) :
        raise RuntimeError("Unable to find resolution element")

    resolution = int(filename_parts[part_index])

    ## The file index string is the next element that is only digits

    part_index += 1

    found_file_index = False

    while ((not found_file_index) and (part_index < num_parts)) :
        if (filename_parts[part_index].isdigit()) :
            found_file_index = True
        else :
            part_index += 1

    if (not found_file_index) :
        raise RuntimeError("Unable to find file index element")

    file_index = int(filename_parts[part_index])

    ## The flame speed is the numeric component that begins with "s"

    part_index = int(0)

    found_flame_speed = False

    while ((not found_flame_speed) and (part_index < num_parts)) :
        
        part = filename_parts[part_index]

        if (part[0] == "s") :
            if (part[1:].isdigit()) :
                found_flame_speed = True

        if (not found_flame_speed) :
            part_index += 1

    if (not found_flame_speed) :
        raise RuntimeError("Unable to find flame speed element")

    flame_speed = float(filename_parts[part_index][1:])

    
    ## Open the file and parse the contents for the remaining elements
    ## ... WHAT IF BOX COORDINATES ARE PRESENT???

    sim_time = None

    box_width = None
    threshold_value = None

    active_boxes = None
    boxes_above = None
    boxes_below = None
    transition_boxes = None

    f = open(filename, "r")

    finished = False

    while (not finished) :

        raw_line = f.readline()

        if (raw_line == "") :

            finished = True

        else :

            line_words = raw_line.split()

            num_words = len(line_words)

            if (num_words > 0) :
                
                keyword = line_words[0]

                if (keyword == "SimTime") :

                    if (num_words != 2) :
                        raise RuntimeError("Improper SimTime entry")

                    sim_time = float(line_words[1])

                elif (keyword == "LengthScale") :

                    if (num_words != 2) :
                        raise RuntimeError("Improper LengthScale entry")

                    box_width = float(line_words[1])

                elif (keyword == "Threshold") :

                    if (num_words != 2) :
                        raise RuntimeError("Improper Threshold entry")

                    threshold_value = float(line_words[1])

                elif (keyword == "ActiveBoxes") :

                    if (num_words != 2) :
                        raise RuntimeError("Improper ActiveBoxes entry")

                    active_boxes = int(line_words[1])

                elif (keyword == "AboveBoxes") :

                    if (num_words != 2) :
                        raise RuntimeError("Improper AboveBoxes entry")

                    boxes_above = int(line_words[1])

                elif (keyword == "BelowBoxes") :

                    if (num_words != 2) :
                        raise RuntimeError("Improper BelowBoxes entry")

                    boxes_below = int(line_words[1])

                elif (keyword == "TransitionBoxes") :

                    if (num_words != 2) :
                        raise RuntimeError("Improper TransitionBoxes entry")

                    transition_boxes = int(line_words[1])

        
    ## Check the entries

    if (sim_time is None) :
        raise RuntimeError("SimTime entry not found")

    if (box_width is None) :
        raise RuntimeError("LengthScale entry not found")

    if (threshold_value is None) :
        raise RuntimeError("Threshold entry not found")

    if (active_boxes is None) :
        raise RuntimeError("ActiveBoxes entry not found")

    if (boxes_above is None) :
        raise RuntimeError("AboveBoxes entry not found")

    if (boxes_below is None) :
        raise RuntimeError("BelowBoxes entry not found")

    if (transition_boxes is None) :
        raise RuntimeError("TransitionBoxes entry not found")


    ## Build the BoxcountInfo object

    boxcount_info = BoxcountInfo()

    boxcount_info.reset(resolution, flame_speed, file_index, sim_time,
                        box_width, threshold_value, active_boxes,
                        boxes_above, boxes_below, transition_boxes)

    return boxcount_info




