// C++ header file cyl_domain.hpp

/*
  By Nathan C. Hearn
     February 7, 2007

  Cylindrical domain for sampling.
*/


#ifndef CYL_DOMAIN_HPP
#define CYL_DOMAIN_HPP


#include <vector>


class CylDomain
  {
  public:
    CylDomain(const std::vector<double> & center_coord, const double radius) :
      center(center_coord), sqRad(radius * radius)
      { }
    ~CylDomain() { }

    bool in_domain(const std::vector<double> & coord) const
      {
      bool inside = false;

      double coord_sqRad = 0.0;

      // Ignore x-coordinate

      const unsigned int dims = center.size();

      for (unsigned int i = 1; i < dims; i++)
	{
	const double axis_dist = coord[i] - center[i];

	coord_sqRad += axis_dist * axis_dist;
	}

      if (coord_sqRad < sqRad)
	inside = true;

      return inside;
      }
      
  private:
    const std::vector<double> center;
    const double sqRad;
  };


#endif  // CYL_DOMAIN_HPP
