#!/usr/bin/env python


try :
    import psyco
    psyco.full()
except ImportError :
    pass


from binreader import TimestepList


png_format_suffix = ".png"
eps_format_suffix = ".eps"

png_pylab_type = "tkAgg"
eps_pylab_type = "PS"

format_suffix = eps_format_suffix
pylab_type = eps_pylab_type

left_layer_bound = 0.80
left_below = True
right_layer_bound = 0.20
right_below = False

x_min = -0.03
x_max = 0.03
y_min = -0.1
y_max = 1.1


def main(arg_list) :

    from sys import stderr, exit

    argc = len(arg_list)

    if (argc < 5) :
        stderr.write("Usage: " + argv[0] + " file graph_title "
                     + "bubble_limit spike_limit [ output_file_prefix ]\n")
        exit()

    argPtr = 1

    filename = str(argv[argPtr])
    argPtr += 1

    graph_title = str(argv[argPtr])
    argPtr += 1

    bubble_limit = float(argv[argPtr])
    argPtr += 1

    spike_limit = float(argv[argPtr])
    argPtr += 1

    save_fig = False

    output_file_prefix = None

    if (argPtr < argc) :
        output_file_prefix = str(argv[argPtr])
        argPtr += 1

        save_fig = True

    # Initialize matplotlib

    import matplotlib
    matplotlib.use(pylab_type)

    import pylab


    # Load the bin data from binrt

    ts_list = TimestepList(filename)

    num_lists = len(ts_list)

    # Run through the timesteps, plotting profiles and measuring bubbles/spikes

    time_list = list()
    
    bubble_lengths_orig = list()
    spike_lengths_orig = list()
    
    bubble_lengths_mapped = list()
    spike_lengths_mapped = list()

    for ts_index in range(num_lists) :
        
        current_ts = ts_list[ts_index]

        current_time = current_ts.get_time() * 1.0e+09  # ns
        time_list.append(current_time)

        stderr.write("Current time [ " + str(current_time) + " ns ]\n")

        orig_coords = current_ts.get_bin_center_align()
        map_coords = current_ts.get_map_bin_center_align()

        bubble_index = current_ts.find_transition_min_index(bubble_limit)

        if (bubble_index is None) :
            raise ValueError("Bubble could not be found")

        bubble_orig_pos = orig_coords[bubble_index]
        bubble_map_pos = map_coords[bubble_index]
        
        bubble_lengths_orig.append(bubble_orig_pos)
        bubble_lengths_mapped.append(bubble_map_pos)

        spike_index = current_ts.find_transition_max_index(spike_limit)

        if (spike_index is None) :
            raise ValueError("Spike could not be found")

        spike_orig_pos = orig_coords[spike_index]
        spike_map_pos = map_coords[spike_index]
        
        spike_lengths_orig.append(spike_orig_pos)
        spike_lengths_mapped.append(spike_map_pos)


        fig = pylab.figure(1)
        fig.clf()
        fig.hold(True)
        
        # mean_fig = fig.add_subplot(2,1,1)
        # stddev_fig = fig.add_subplot(2,1,2)

        mean_fig = fig.add_subplot(1,1,1)


        # Determine the extent of the coordinates

        means = current_ts.get_bin_mean()
        stddevs = current_ts.get_bin_stddev()
        
        mins = current_ts.get_bin_min()
        maxs = current_ts.get_bin_max()

        num_bins = current_ts.get_num_bins()

        mean_pstd = list()
        mean_mstd = list()

        for i in range(num_bins) :
            stddev_val = stddevs[i]
            mean_val = means[i]
            
            mean_pstd.append(mean_val + stddev_val)
            mean_mstd.append(mean_val - stddev_val)

        mean_fig.set_title(graph_title + " - "
                           + str(current_time) + " ns")

        bubble_box_orig_x = [ 0.0, bubble_orig_pos, bubble_orig_pos, 0.0 ]
        bubble_box_map_x = [ 0.0, bubble_map_pos, bubble_map_pos, 0.0 ]

        bubble_box_y = [ 0.5, 0.5, bubble_limit, bubble_limit ]

        mean_fig.fill(bubble_box_orig_x, bubble_box_y,
                      label="Original Bubble Domain", alpha=1.0,
                      linewidth=0.0, facecolor="magenta")

        mean_fig.fill(bubble_box_map_x, bubble_box_y,
                      label="Corrected Bubble Domain", alpha=1.0,
                      linewidth=0.0, facecolor="teal")

        spike_box_orig_x = [ 0.0, spike_orig_pos, spike_orig_pos, 0.0 ]
        spike_box_map_x = [ 0.0, spike_map_pos, spike_map_pos, 0.0 ]

        spike_box_y = [ 0.5, 0.5, spike_limit, spike_limit ]

        mean_fig.fill(spike_box_orig_x, spike_box_y,
                      label="Original Spike Domain", alpha=1.0,
                      linewidth=0.0, facecolor="cyan")

        mean_fig.fill(spike_box_map_x, spike_box_y,
                      label="Corrected Spike Domain", alpha=1.0,
                      linewidth=0.0, facecolor="yellow")

        # mean_fig.plot((0.0, bubble_orig_pos), (bubble_limit, bubble_limit),
        #               label="Original Bubble Length", linewidth=2, alpha=0.5)
        # mean_fig.plot((0.0, bubble_map_pos), (bubble_limit, bubble_limit),
        #               label="Corrected Bubble Length", linewidth=2,
        #               alpha=0.5)
        
        # mean_fig.plot((0.0, spike_orig_pos), (spike_limit, spike_limit),
        #               label="Original Spike Length", linewidth=2, alpha=0.5)
        # mean_fig.plot((0.0, spike_map_pos), (spike_limit, spike_limit),
        #               label="Corrected Spike Length", linewidth=2, alpha=0.5)
        
        # mean_fig.plot(map_coords, mean_pstd, label="Corrected Upper StdDev")
        # mean_fig.plot(map_coords, mean_mstd, label="Corrected Lower Stddev")

        # stddev_fig.plot(orig_coords, stddevs, label="Original StdDev")
        # stddev_fig.plot(map_coords, stddevs, label="Corrected StdDev")

        mean_fig.plot(orig_coords, means, "r", label="Original Mean",
                      linewidth=2.0)
        mean_fig.plot(map_coords, means, "g", label="Corrected Mean",
                      linewidth=2.0)

        mean_fig.set_xlim(x_min, x_max)
        # stddev_fig.set_xlim(x_min, x_max)
        mean_fig.set_ylim(y_min, y_max)
        mean_fig.legend(loc="lower left")
        mean_fig.grid(True)
        # stddev_fig.grid(True)

        # stddev_fig.set_xlabel("Position (cm) - "
        #                       + "Relative to Unperturbed Interface")
        mean_fig.set_xlabel("Position (cm) - "
                            + "Relative to Unperturbed Interface")
        mean_fig.set_ylabel("Polyimide Mass Fraction")
        

        if (save_fig) :
            output_filename = output_file_prefix + "_" \
                              + ("%(index)04d" % { "index" : ts_index}) \
                              + format_suffix

            pylab.savefig(output_filename)
        else :
            pylab.show()

    # Plot the bubble and spike lengths over time

    width_plot = pylab.figure()
    width_plot.clf()
    width_plot.hold(True)

    width_fig = width_plot.add_subplot(1,1,1)

    width_fig.set_title(graph_title + " - Spike and Bubble Positions")

    width_fig.plot(time_list, bubble_lengths_orig, label="Original Bubble")
    width_fig.plot(time_list, bubble_lengths_mapped, label="Corrected Bubble")

    width_fig.plot(time_list, spike_lengths_orig, label="Original Spike")
    width_fig.plot(time_list, spike_lengths_mapped, label="Corrected Spike")

    width_fig.set_xlabel("Time (ns)")
    width_fig.set_ylabel("Relative Position (cm)")

    width_fig.grid(True)
    width_fig.legend(loc="best")

    if (save_fig) :
        output_filename = output_file_prefix + "_sblengths" + format_suffix
        pylab.savefig(output_filename)

    if (save_fig) :
        
        # Save the spike and bubble lengths over time

        outfile = file(output_file_prefix + "_sblengths.csv", "w")

        outfile.write("#Time Orig_Bubble_Length Map_Bubble_Length "
                      + "Orig_Spike_Length Map_Spike_Length\n")

        for i in range(num_lists) :
            outfile.write(str(time_list[i]) + " " + str(bubble_lengths_orig[i])
                          + " " + str(bubble_lengths_mapped[i]) + " "
                          + str(spike_lengths_orig[i]) + " "
                          + str(spike_lengths_mapped[i]) + "\n")


    if (not save_fig) :
        pylab.show()
            

if (__name__ == "__main__") :

    from sys import argv

    main(argv)
    
