// C++ program file map_pos.cpp

/*
  By Nathan C. Hearn
     January 20, 2007

  Maps a set of 1D time-position pairs to a reference position.
*/


#include <string>
#include <vector>
#include <iostream>
#include <fstream>
#include "quickflash.hpp"
#include "part_map.hpp"



void read_time_pos(const std::string & filename, std::vector<double> & time,
		   std::vector<double> & pos)
  {
  // Open the file

  std::ifstream in_file(filename.c_str());

  // Read the number of pairs

  unsigned int num_pairs = 0;

  in_file >> num_pairs;

  // Read in the data

  time.resize(num_pairs);
  pos.resize(num_pairs);

  for (unsigned int pair_index = 0; pair_index < num_pairs; pair_index++)
    {
    in_file >> time[pair_index];
    in_file >> pos[pair_index];
    }
  }


void print_map_pos(const double map_time,
		   const std::vector<double> & orig_time,
		   const std::vector<double> & orig_pos,
		   const std::vector<double> & map_pos,
		   const std::vector<double> & interface_pos)
  {
  using std::cout;
  using std::endl;

  const unsigned int num_pos = map_pos.size();

  if ((orig_time.size() != num_pos) || (orig_pos.size() != num_pos))
    throw QuickFlash::Except("Incompatible vector lengths", __FILE__, __LINE__);

  // Print out the number of entries
  
  cout << num_pos << endl;

  // Print the data

  for (unsigned int index = 0; index < num_pos; index++)
    {
    cout << orig_time[index] << " " << orig_pos[index] << " " << map_time
	 << " " << map_pos[index] << " " << interface_pos[index] << endl;
    }
  }


int main(int argc, char * argv[])
  {
  using std::cerr;
  using std::endl;

  int req_args = 3;

  if (argc != req_args)
    {
    cerr << "Usage: " << argv[0] << " map_file time_pos_file" << endl;
    return 1;
    }

  int argPtr = 0;

  const std::string map_filename = argv[argPtr++];

  const std::string time_pos_filename = argv[argPtr++];

  // Get the map information

  PartMap partmap(map_filename);

  // Get the time-pos pairs

  std::vector<double> orig_time;
  std::vector<double> orig_pos;

  read_time_pos(time_pos_filename, orig_time, orig_pos);

  // Get the new positions

  const unsigned int num_pos = orig_pos.size();

  std::vector<double> map_pos(num_pos);
  std::vector<double> interface_pos(num_pos);

  for (unsigned int index = 0; index < num_pos; index++)
    {
    const double pos_time = orig_time[index];

    map_pos[index] = partmap.get_initial_position(pos_time, orig_pos[index]);

    interface_pos[index] = partmap.get_interface_position(pos_time);
    }

  const double map_time = partmap.get_map_time();

  print_map_pos(map_time, orig_time, orig_pos, map_pos, interface_pos);

  return 0;
  }
