// C++ header file mixing_layer.hpp

/*
  By Nathan C. Hearn
     May 16, 2007

  Classes for storing mixing layer information for each timestep.
*/


#ifndef MIXING_LAYER_HPP
#define MIXING_LAYER_HPP


#include <string>
#include <vector>


class MixingLayer
  {
  public:
    MixingLayer() : time(0.0), min_pos(0.0), max_pos(0.0), int_pos(0.0) { }

    MixingLayer(const double sim_time, const double minimum_pos, 
		const double maximum_pos, const double interface_pos) :
      time(0.0), min_pos(0.0), max_pos(0.0), int_pos(0.0) 
      { reset(sim_time, minimum_pos, maximum_pos, interface_pos); }

    MixingLayer(const MixingLayer & source) :
      time(0.0), min_pos(0.0), max_pos(0.0), int_pos(0.0) 
      { reset(source); }

    ~MixingLayer() { }

    MixingLayer & operator=(const MixingLayer & source)
      {
      reset(source);
      return *this;
      }

    bool operator<(const double ref_time)
      { return (time < ref_time); }

    bool operator<(const MixingLayer & source)
      { return (time < source.time); }

    double get_time() const { return time; }

    double get_min_pos() const { return min_pos; }

    double get_max_pos() const { return max_pos; }

    double get_int_pos() const { return int_pos; }

    void reset()
      {
      time = 0.0;

      min_pos = 0.0;
      max_pos = 0.0;

      int_pos = 0.0;
      }
      
    void reset(const double sim_time, const double minimum_pos, 
	       const double maximum_pos, const double interface_pos)
      {
      time = sim_time;

      min_pos = minimum_pos;
      max_pos = maximum_pos;

      int_pos = interface_pos;
      }

    void reset(const MixingLayer & source)
      {
      if (&source != this)
	{
	time = source.time;

	min_pos = source.min_pos;
	max_pos = source.max_pos;

	int_pos = source.int_pos;
	}
      }

  private:
    double time;

    double min_pos;
    double max_pos;
    double int_pos;
  };


void get_mixing_layer(const std::string & flash_filename, 
		      const double min_x, const double max_x, 
		      const double bin_spacing, 
		      const double min_poly, const double max_poly,
		      MixingLayer & mixing_layer,
		      const unsigned int buffer_size=1000);

void get_mixing_layers(const std::vector<std::string> & flash_filename_list,
		       const double min_x, const double max_x, 
		       const double bin_spacing, 
		       const double min_poly, const double max_poly,
		       std::vector<MixingLayer> & mixing_layer_list,
		       const unsigned int buffer_size=1000);

void get_mixing_layer_minmax(const std::string & flash_filename, 
			     const double min_poly, const double max_poly,
			     MixingLayer & mixing_layer,
			     const unsigned int buffer_size=1000);

void get_mixing_layers_minmax(const 
			      std::vector<std::string> & flash_filename_list,
			      const double min_poly, const double max_poly,
			      std::vector<MixingLayer> & mixing_layer_list,
			      const unsigned int buffer_size=1000);

#endif  // MIXING_LAYER_HPP
