// C++ header file moving_boundary.hpp

/*
  By Nathan C. Hearn
     February 7, 2007

  Moving boundary definition.
*/


#ifndef MOVING_BOUNDARY_HPP
#define MOVING_BOUNDARY_HPP


#include <vector>


class MovingBoundary
  {
  public:
    MovingBoundary(const std::vector<double> & coefficients) :
      coeff(coefficients)
      { }
    MovingBoundary(const unsigned int num_elems, const double coefficients[]) :
      coeff()
      {
      coeff.resize(num_elems);

      for (unsigned int i = 0; i < num_elems; i++)
	coeff[i] = coefficients[i];
      }
    ~MovingBoundary() { }

    double operator[](const double time) const
      {
      double coord = 0.0;

      const unsigned int num_coeff = coeff.size();

      for (unsigned int i = (num_coeff-1); i > 0; i--)
	{
	coord += coeff[i];
	coord *= time;
	}

      if (num_coeff > 0)
	coord += coeff[0];

      return coord;
      }

  private:
    std::vector<double> coeff;
  };


#endif  // MOVING_BOUNDARY_HPP
