# composite.py

# By Nathan C. Hearn
#    November 22, 2007


import SciCompositor as scicomp


class SliceComposite :

    def __init__(self, main_title, time, flame_pos, slice_min_coords,
                 slice_max_coords, xsect_bottom_pos, xsect_flame_front_pos,
                 xsect_top_pos, xsect_min_coords, xsect_max_coords) :

        self.__main_title = main_title
        self.__time = time
        self.__flame_pos = flame_pos

        self.__slice_min_x = slice_min_coords[0]
        self.__slice_max_x = slice_max_coords[0]

        self.__slice_min_y = slice_min_coords[1]
        self.__slice_max_y = slice_max_coords[1]

        self.__slice_width = self.__slice_max_x - self.__slice_min_x
        self.__slice_height = self.__slice_max_y - self.__slice_min_y

        self.__slice_aspect_h_w = self.__slice_height / self.__slice_width

        self.__xsect_min_x = xsect_min_coords[0]
        self.__xsect_max_x = xsect_max_coords[0]

        self.__xsect_min_y = xsect_min_coords[1]
        self.__xsect_max_y = xsect_max_coords[1]

        self.__xsect_width = self.__xsect_max_x - self.__xsect_min_x
        self.__xsect_height = self.__xsect_max_y - self.__xsect_min_y

        self.__xsect_aspect_h_w = self.__xsect_height / self.__xsect_width

        self.__xsect_bottom_pos = xsect_bottom_pos
        self.__xsect_flame_pos = xsect_flame_front_pos
        self.__xsect_top_pos = xsect_top_pos

        self.__dataset_titles = list()
        self.__dataset_min_values = list()
        self.__dataset_max_values = list()

        self.__xy_slice_image_files = list()
        self.__xy_slice_min_values = list()
        self.__xy_slice_max_values = list()
        
        self.__xz_slice_image_files = list()
        self.__xz_slice_min_values = list()
        self.__xz_slice_max_values = list()

        self.__bottom_xsect_image_files = list()
        self.__bottom_xsect_min_values = list()
        self.__bottom_xsect_max_values = list()

        self.__flame_xsect_image_files = list()
        self.__flame_xsect_min_values = list()
        self.__flame_xsect_max_values = list()

        self.__top_xsect_image_files = list()
        self.__top_xsect_min_values = list()
        self.__top_xsect_max_values = list()


    def add_dataset(self, dataset_title, dataset_min_value, dataset_max_value,
                    xy_image_file, xy_slice_min_value, xy_slice_max_value,
                    xz_image_file, xz_slice_min_value, xz_slice_max_value,
                    bottom_xsect_image_file, 
                    bottom_xsect_min_value, bottom_xsect_max_value,
                    flame_front_xsect_image_file, 
                    flame_front_xsect_min_value, flame_front_xsect_max_value,
                    top_xsect_image_file, 
                    top_xsect_min_value, top_xsect_max_value) :

        self.__dataset_titles.append(dataset_title)

        self.__dataset_min_values.append(dataset_min_value)
        self.__dataset_max_values.append(dataset_max_value)

        self.__xy_slice_image_files.append(xy_image_file)

        self.__xy_slice_min_values.append(xy_slice_min_value)
        self.__xy_slice_max_values.append(xy_slice_max_value)

        self.__xz_slice_image_files.append(xz_image_file)

        self.__xz_slice_min_values.append(xz_slice_min_value)
        self.__xz_slice_max_values.append(xz_slice_max_value)

        self.__bottom_xsect_image_files.append(bottom_xsect_image_file)
        self.__bottom_xsect_min_values.append(bottom_xsect_min_value)
        self.__bottom_xsect_max_values.append(bottom_xsect_max_value)

        self.__flame_xsect_image_files.append(flame_front_xsect_image_file)
        self.__flame_xsect_min_values.append(flame_front_xsect_min_value)
        self.__flame_xsect_max_values.append(flame_front_xsect_max_value)

        self.__top_xsect_image_files.append(top_xsect_image_file)
        self.__top_xsect_min_values.append(top_xsect_min_value)
        self.__top_xsect_max_values.append(top_xsect_max_value)


    def generate_svg(self, canvas_width) :

        from SciCompositor import svg_defs

        num_datasets = len(self.__dataset_titles)

        slice_port_image_width = 0.85 * canvas_width
        
        slice_port_image_height \
            = self.__slice_aspect_h_w * slice_port_image_width

        xsect_port_height = slice_port_image_height
        
        xsect_port_width = xsect_port_height / self.__xsect_aspect_h_w

        dataset_port_spacing = 4.75 * slice_port_image_height

        canvas_height = (float(num_datasets) + 0.2) * dataset_port_spacing

        header_start_x = 0.01 * canvas_width
        header_start_y = canvas_height - (0.07 * dataset_port_spacing)

        header_end_x = 0.9 * canvas_width
        header_end_y = header_start_y + (0.05 * dataset_port_spacing)

        header_width = header_end_x - header_start_x
        header_height = header_end_y - header_start_y

        main_title_font_size = 0.75 * (header_end_y - header_start_y)
        
        dataset_title_start_x = 0.01 * canvas_width
        dataset_title_start_y = 0.95 * dataset_port_spacing

        dataset_area_start_x = 0.0
        dataset_area_start_y = canvas_height - (1.15 * dataset_port_spacing)

        dataset_title_font_size = 0.05 * dataset_port_spacing
        dataset_desc_font_size = 0.03 * dataset_port_spacing

        dataset_desc_start_x = 0.9 * canvas_width
        dataset_desc_start_y = dataset_title_start_y

        dataset_desc_text_anchor = svg_defs.svg_text_anchor_end

        slice_desc_font_size = 0.015 * dataset_port_spacing

        slice_port_desc_start_x = 0.0 * canvas_width
        slice_port_desc_start_y = -0.3 * slice_port_image_height

        xy_slice_port_image_start_x = 0.1 * canvas_width
        xy_slice_port_image_start_y = 0.72 * dataset_port_spacing

        xy_slice_port_image_end_x = xy_slice_port_image_start_x \
                                    + slice_port_image_width
        
        xy_slice_port_image_end_y = xy_slice_port_image_start_y \
                                    + slice_port_image_height

        xz_slice_port_image_start_x = 0.1 * canvas_width
        xz_slice_port_image_start_y = 0.42 * dataset_port_spacing

        xz_slice_port_image_end_x = xz_slice_port_image_start_x \
                                    + slice_port_image_width
        
        xz_slice_port_image_end_y = xz_slice_port_image_start_y \
                                    + slice_port_image_height

        slice_axes_font_size = 0.1 * self.__slice_height

        slice_axes_x_ticks = 10
        slice_axes_y_ticks = 5

        xsect_axes_x_ticks = 1
        xsect_axes_y_ticks = 5

        xsect_port_start_y = 0.1 * dataset_port_spacing
        xsect_port_end_y = xsect_port_start_y + xsect_port_height

        xsect_desc_start_x = 1.1 * xsect_port_width

        xsect_desc_title_start_y = 0.8 * xsect_port_height
        xsect_desc_minval_start_y = 0.5 * xsect_port_height
        xsect_desc_maxval_start_y = 0.2 * xsect_port_height

        xsect_desc_font_size = 0.75 * dataset_desc_font_size
        
        xsect_bottom_port_start_x = 0.1 * canvas_width
        xsect_bottom_port_end_x = xsect_bottom_port_start_x + xsect_port_width
        
        xsect_flame_port_start_x = 0.43 * canvas_width
        xsect_flame_port_end_x = xsect_flame_port_start_x + xsect_port_width
        
        xsect_top_port_start_x = 0.76 * canvas_width
        xsect_top_port_end_x = xsect_top_port_start_x + xsect_port_width
        
        # Create the canvas

        canvas = scicomp.Canvas(0.0, canvas_width, 0.0, canvas_height)

        # Build the header

        header_port = canvas.get_new_viewport("header_port",
                                              header_start_x, header_end_x,
                                              header_start_y, header_end_y,
                                              0.0, header_width,
                                              0.0, header_height)

        header_text_string = self.__main_title \
                             + " - Time " + str(self.__time) \
                             + " - Estimated flame position " \
                             + str(self.__flame_pos)

        header_text = scicomp.TextObject("header_text", header_text_string)

        header_text.set_anchor(x=0.0, y=0.0, font_size=main_title_font_size)

        header_port.add_child_object(header_text)

        # Run through the datasets

        for set_index in range(num_datasets) :

            dataset_title = self.__dataset_titles[set_index]

            dataset_min_value = self.__dataset_min_values[set_index]
            dataset_max_value = self.__dataset_max_values[set_index]

            xy_slice_image_file = self.__xy_slice_image_files[set_index]
            
            xy_slice_min_value = self.__xy_slice_min_values[set_index]
            xy_slice_max_value = self.__xy_slice_max_values[set_index]

            xz_slice_image_file = self.__xz_slice_image_files[set_index]
            
            xz_slice_min_value = self.__xz_slice_min_values[set_index]
            xz_slice_max_value = self.__xz_slice_max_values[set_index]

            xsect_bottom_image_file \
                = self.__bottom_xsect_image_files[set_index]

            xsect_bottom_min_value = self.__bottom_xsect_min_values[set_index]
            xsect_bottom_max_value = self.__bottom_xsect_max_values[set_index]

            xsect_flame_image_file \
                = self.__flame_xsect_image_files[set_index]

            xsect_flame_min_value = self.__flame_xsect_min_values[set_index]
            xsect_flame_max_value = self.__flame_xsect_max_values[set_index]

            xsect_top_image_file \
                = self.__top_xsect_image_files[set_index]

            xsect_top_min_value = self.__top_xsect_min_values[set_index]
            xsect_top_max_value = self.__top_xsect_max_values[set_index]

            # Set up the port for slice information and plot

            dataset_port_min_x = 0.0
            dataset_port_max_x = canvas_width

            dataset_port_min_y = dataset_area_start_y
            dataset_port_max_y = dataset_area_start_y + dataset_port_spacing

            dataset_port \
                = canvas.get_new_viewport((dataset_title + "_port"),
                                          dataset_port_min_x,
                                          dataset_port_max_x,
                                          dataset_port_min_y,
                                          dataset_port_max_y,
                                          0.0, canvas_width,
                                          0.0, dataset_port_spacing)

            # Write the title

            title_text_string = "Dataset " + dataset_title

            title_text = scicomp.TextObject((dataset_title + "_title"),
                                            title_text_string)

            title_text.set_anchor(x=dataset_title_start_x,
                                  y=dataset_title_start_y,
                                  font_size=dataset_title_font_size)

            dataset_port.add_child_object(title_text)

            # Write the descripton

            dataset_desc_text_string = "Dataset Min Value [ " \
                                       + str(dataset_min_value) \
                                       + " ] Max Value [ " \
                                       + str(dataset_max_value) \
                                       + " ]"

            dataset_desc_text = scicomp.TextObject((dataset_title + "_desc"),
                                                   dataset_desc_text_string)

            dataset_desc_text.set_anchor(x=dataset_desc_start_x,
                                         y=dataset_desc_start_y,
                                         font_size=dataset_desc_font_size,
                                         anchor_pos=dataset_desc_text_anchor)
            
            dataset_port.add_child_object(dataset_desc_text)

            # Build the xy Plot port

            xy_plot_port \
                = dataset_port.get_sub_viewport((dataset_title + "_xy_plot"),
                                                xy_slice_port_image_start_x,
                                                xy_slice_port_image_end_x,
                                                xy_slice_port_image_start_y,
                                                xy_slice_port_image_end_y,
                                                0.0, slice_port_image_width,
                                                0.0, slice_port_image_height)

            xy_plot_port.set_physical_coord_map(self.__slice_min_x,
                                                self.__slice_max_x,
                                                self.__slice_min_y,
                                                self.__slice_max_y)

            xy_plot_img = scicomp.ImageObject((dataset_title + "_xy_img"),
                                              xy_slice_image_file,
                                              self.__slice_min_x,
                                              self.__slice_max_x,
                                              self.__slice_min_y,
                                              self.__slice_max_y,
                                              use_physical_coords=True)

            xy_plot_port.add_child_object(xy_plot_img)

            xy_plot_axes = scicomp.AxesObject((dataset_title + "_xy_axes"),
                                              self.__slice_min_x,
                                              self.__slice_max_x,
                                              self.__slice_min_y,
                                              self.__slice_max_y,
                                              use_physical_coords=True)

            xy_plot_axes.set_frame_color(red=1.0, green=0.0, blue=0.0)
            xy_plot_axes.set_frame_line_width(1.0)

            x_ticks = slice_axes_x_ticks
            y_ticks = slice_axes_y_ticks
            
            xy_plot_axes.set_x_major_ticks_regular(max_ticks=x_ticks)
            xy_plot_axes.set_y_major_ticks_regular(max_ticks=y_ticks)

            xy_plot_axes.set_major_ticks_color(red=0.7, green=0.0, blue=0.7,
                                               opacity=0.5)
            xy_plot_axes.set_major_ticks_line_width(1.0)

            xy_plot_axes.set_axis_label_font_size(slice_axes_font_size)

            xy_plot_port.add_child_object(xy_plot_axes)

            # Write the descripton

            xy_desc_text_string = "x-y Plane - Slice Min Value [ " \
                                  + str(xy_slice_min_value) \
                                  + " ] Max Value [ " \
                                  + str(xy_slice_max_value) \
                                  + " ]"

            xy_desc_text = scicomp.TextObject((dataset_title + "_desc"),
                                              xy_desc_text_string)

            xy_desc_text.set_anchor(x=slice_port_desc_start_x,
                                    y=slice_port_desc_start_y,
                                    font_size=dataset_desc_font_size)
            
            xy_plot_port.add_child_object(xy_desc_text)



            # Build the xz Plot port

            xz_plot_port \
                = dataset_port.get_sub_viewport((dataset_title + "_xz_plot"),
                                                xz_slice_port_image_start_x,
                                                xz_slice_port_image_end_x,
                                                xz_slice_port_image_start_y,
                                                xz_slice_port_image_end_y,
                                                0.0, slice_port_image_width,
                                                0.0, slice_port_image_height)

            xz_plot_port.set_physical_coord_map(self.__slice_min_x,
                                                self.__slice_max_x,
                                                self.__slice_min_y,
                                                self.__slice_max_y)

            xz_plot_img = scicomp.ImageObject((dataset_title + "_xz_img"),
                                              xz_slice_image_file,
                                              self.__slice_min_x,
                                              self.__slice_max_x,
                                              self.__slice_min_y,
                                              self.__slice_max_y,
                                              use_physical_coords=True)

            xz_plot_port.add_child_object(xz_plot_img)

            xz_plot_axes = scicomp.AxesObject((dataset_title + "_xz_axes"),
                                              self.__slice_min_x,
                                              self.__slice_max_x,
                                              self.__slice_min_y,
                                              self.__slice_max_y,
                                              use_physical_coords=True)

            xz_plot_axes.set_frame_color(red=1.0, green=0.0, blue=0.0)
            xz_plot_axes.set_frame_line_width(1.0)

            x_ticks = slice_axes_x_ticks
            y_ticks = slice_axes_y_ticks
            
            xz_plot_axes.set_x_major_ticks_regular(max_ticks=x_ticks)
            xz_plot_axes.set_y_major_ticks_regular(max_ticks=y_ticks)

            xz_plot_axes.set_major_ticks_color(red=0.7, green=0.0, blue=0.7,
                                               opacity=0.5)
            xz_plot_axes.set_major_ticks_line_width(1.0)

            xz_plot_axes.set_axis_label_font_size(slice_axes_font_size)

            xz_plot_port.add_child_object(xz_plot_axes)

            # Write the descripton

            xz_desc_text_string = "x-z Plane - Slice Min Value [ " \
                                  + str(xz_slice_min_value) \
                                  + " ] Max Value [ " \
                                  + str(xz_slice_max_value) \
                                  + " ]"

            xz_desc_text = scicomp.TextObject((dataset_title + "_desc"),
                                              xz_desc_text_string)

            xz_desc_text.set_anchor(x=slice_port_desc_start_x,
                                    y=slice_port_desc_start_y,
                                    font_size=dataset_desc_font_size)
            
            xz_plot_port.add_child_object(xz_desc_text)


            # Build the bottom cross-section plot port

            xsect_bottom_port \
                = dataset_port.get_sub_viewport((dataset_title
                                                 + "_xsect_bottom_plot"),
                                                xsect_bottom_port_start_x,
                                                xsect_bottom_port_end_x,
                                                xsect_port_start_y,
                                                xsect_port_end_y,
                                                0.0, xsect_port_width,
                                                0.0, xsect_port_height)

            xsect_bottom_port.set_physical_coord_map(self.__xsect_min_x,
                                                     self.__xsect_max_x,
                                                     self.__xsect_min_y,
                                                     self.__xsect_max_y)

            xsect_bottom_img = scicomp.ImageObject((dataset_title
                                                    + "_xsect_bottom_img"),
                                                   xsect_bottom_image_file,
                                                   self.__xsect_min_x,
                                                   self.__xsect_max_x,
                                                   self.__xsect_min_y,
                                                   self.__xsect_max_y,
                                                   use_physical_coords=True)

            xsect_bottom_port.add_child_object(xsect_bottom_img)

            xsect_bottom_axes = scicomp.AxesObject((dataset_title
                                                    + "_xsect_bottom_axes"),
                                                   self.__xsect_min_x,
                                                   self.__xsect_max_x,
                                                   self.__xsect_min_y,
                                                   self.__xsect_max_y,
                                                   use_physical_coords=True)

            xsect_bottom_axes.set_frame_color(red=1.0, green=0.0, blue=0.0)
            xsect_bottom_axes.set_frame_line_width(1.0)

            x_ticks = xsect_axes_x_ticks
            y_ticks = xsect_axes_y_ticks
            
            xsect_bottom_axes.set_x_major_ticks_regular(max_ticks=x_ticks)
            xsect_bottom_axes.set_y_major_ticks_regular(max_ticks=y_ticks)

            xsect_bottom_axes.set_major_ticks_color(red=0.7, green=0.0,
                                                    blue=0.7, opacity=0.5)
            xsect_bottom_axes.set_major_ticks_line_width(1.0)

            xsect_bottom_axes.set_axis_label_font_size(slice_axes_font_size)

            xsect_bottom_port.add_child_object(xsect_bottom_axes)

            # Write the descripton

            xsect_bottom_title_string = "y-z Plane at x = " \
                                        + str(self.__xsect_bottom_pos)

            xsect_bottom_minval_string = "Min Value [ " \
                                         + str(xsect_bottom_min_value) + " ]"

            xsect_bottom_maxval_string = "Max Value [ " \
                                         + str(xsect_bottom_max_value) + " ]"

            xsect_bottom_title_text \
                = scicomp.TextObject((dataset_title + "_desc_title"),
                                     xsect_bottom_title_string)

            xsect_bottom_title_text.set_anchor(x=xsect_desc_start_x,
                                               y=xsect_desc_title_start_y,
                                               font_size=xsect_desc_font_size)
            
            xsect_bottom_port.add_child_object(xsect_bottom_title_text)


            xsect_bottom_minval_text \
                = scicomp.TextObject((dataset_title + "_desc_minval"),
                                     xsect_bottom_minval_string)

            xsect_bottom_minval_text.set_anchor(x=xsect_desc_start_x,
                                                y=xsect_desc_minval_start_y,
                                                font_size=xsect_desc_font_size)
            
            xsect_bottom_port.add_child_object(xsect_bottom_minval_text)


            xsect_bottom_maxval_text \
                = scicomp.TextObject((dataset_title + "_desc_maxval"),
                                     xsect_bottom_maxval_string)

            xsect_bottom_maxval_text.set_anchor(x=xsect_desc_start_x,
                                                y=xsect_desc_maxval_start_y,
                                                font_size=xsect_desc_font_size)
            
            xsect_bottom_port.add_child_object(xsect_bottom_maxval_text)


            # Build the flame-front cross-section plot port

            xsect_flame_port \
                = dataset_port.get_sub_viewport((dataset_title
                                                 + "_xsect_flame_plot"),
                                                xsect_flame_port_start_x,
                                                xsect_flame_port_end_x,
                                                xsect_port_start_y,
                                                xsect_port_end_y,
                                                0.0, xsect_port_width,
                                                0.0, xsect_port_height)

            xsect_flame_port.set_physical_coord_map(self.__xsect_min_x,
                                                    self.__xsect_max_x,
                                                    self.__xsect_min_y,
                                                    self.__xsect_max_y)

            xsect_flame_img = scicomp.ImageObject((dataset_title
                                                   + "_xsect_flame_img"),
                                                  xsect_flame_image_file,
                                                  self.__xsect_min_x,
                                                  self.__xsect_max_x,
                                                  self.__xsect_min_y,
                                                  self.__xsect_max_y,
                                                  use_physical_coords=True)

            xsect_flame_port.add_child_object(xsect_flame_img)

            xsect_flame_axes = scicomp.AxesObject((dataset_title
                                                   + "_xsect_flame_axes"),
                                                  self.__xsect_min_x,
                                                  self.__xsect_max_x,
                                                  self.__xsect_min_y,
                                                  self.__xsect_max_y,
                                                  use_physical_coords=True)

            xsect_flame_axes.set_frame_color(red=1.0, green=0.0, blue=0.0)
            xsect_flame_axes.set_frame_line_width(1.0)

            x_ticks = xsect_axes_x_ticks
            y_ticks = xsect_axes_y_ticks
            
            xsect_flame_axes.set_x_major_ticks_regular(max_ticks=x_ticks)
            xsect_flame_axes.set_y_major_ticks_regular(max_ticks=y_ticks)

            xsect_flame_axes.set_major_ticks_color(red=0.7, green=0.0,
                                                    blue=0.7, opacity=0.5)
            xsect_flame_axes.set_major_ticks_line_width(1.0)

            xsect_flame_axes.set_axis_label_font_size(slice_axes_font_size)

            xsect_flame_port.add_child_object(xsect_flame_axes)

            # Write the descripton

            xsect_flame_title_string = "y-z Plane at x = " \
                                       + str(self.__xsect_flame_pos)

            xsect_flame_minval_string = "Min Value [ " \
                                        + str(xsect_flame_min_value) + " ]"

            xsect_flame_maxval_string = "Max Value [ " \
                                        + str(xsect_flame_max_value) + " ]"

            xsect_flame_title_text \
                = scicomp.TextObject((dataset_title + "_desc_title"),
                                     xsect_flame_title_string)

            xsect_flame_title_text.set_anchor(x=xsect_desc_start_x,
                                              y=xsect_desc_title_start_y,
                                              font_size=xsect_desc_font_size)
            
            xsect_flame_port.add_child_object(xsect_flame_title_text)


            xsect_flame_minval_text \
                = scicomp.TextObject((dataset_title + "_desc_minval"),
                                     xsect_flame_minval_string)

            xsect_flame_minval_text.set_anchor(x=xsect_desc_start_x,
                                               y=xsect_desc_minval_start_y,
                                               font_size=xsect_desc_font_size)
            
            xsect_flame_port.add_child_object(xsect_flame_minval_text)


            xsect_flame_maxval_text \
                = scicomp.TextObject((dataset_title + "_desc_maxval"),
                                     xsect_flame_maxval_string)

            xsect_flame_maxval_text.set_anchor(x=xsect_desc_start_x,
                                                y=xsect_desc_maxval_start_y,
                                                font_size=xsect_desc_font_size)
            
            xsect_flame_port.add_child_object(xsect_flame_maxval_text)


            # Build the top cross-section plot port

            xsect_top_port \
                = dataset_port.get_sub_viewport((dataset_title
                                                 + "_xsect_top_plot"),
                                                xsect_top_port_start_x,
                                                xsect_top_port_end_x,
                                                xsect_port_start_y,
                                                xsect_port_end_y,
                                                0.0, xsect_port_width,
                                                0.0, xsect_port_height)

            xsect_top_port.set_physical_coord_map(self.__xsect_min_x,
                                                  self.__xsect_max_x,
                                                  self.__xsect_min_y,
                                                  self.__xsect_max_y)

            xsect_top_img = scicomp.ImageObject((dataset_title
                                                 + "_xsect_top_img"),
                                                xsect_top_image_file,
                                                self.__xsect_min_x,
                                                self.__xsect_max_x,
                                                self.__xsect_min_y,
                                                self.__xsect_max_y,
                                                use_physical_coords=True)

            xsect_top_port.add_child_object(xsect_top_img)

            xsect_top_axes = scicomp.AxesObject((dataset_title
                                                 + "_xsect_top_axes"),
                                                self.__xsect_min_x,
                                                self.__xsect_max_x,
                                                self.__xsect_min_y,
                                                self.__xsect_max_y,
                                                use_physical_coords=True)

            xsect_top_axes.set_frame_color(red=1.0, green=0.0, blue=0.0)
            xsect_top_axes.set_frame_line_width(1.0)

            x_ticks = xsect_axes_x_ticks
            y_ticks = xsect_axes_y_ticks
            
            xsect_top_axes.set_x_major_ticks_regular(max_ticks=x_ticks)
            xsect_top_axes.set_y_major_ticks_regular(max_ticks=y_ticks)

            xsect_top_axes.set_major_ticks_color(red=0.7, green=0.0,
                                                    blue=0.7, opacity=0.5)
            xsect_top_axes.set_major_ticks_line_width(1.0)

            xsect_top_axes.set_axis_label_font_size(slice_axes_font_size)

            xsect_top_port.add_child_object(xsect_top_axes)

            # Write the descripton

            xsect_top_title_string = "y-z Plane at x = " \
                                     + str(self.__xsect_top_pos)

            xsect_top_minval_string = "Min Value [ " \
                                      + str(xsect_top_min_value) + " ]"

            xsect_top_maxval_string = "Max Value [ " \
                                      + str(xsect_top_max_value) + " ]"

            xsect_top_title_text \
                = scicomp.TextObject((dataset_title + "_desc_title"),
                                     xsect_top_title_string)

            xsect_top_title_text.set_anchor(x=xsect_desc_start_x,
                                            y=xsect_desc_title_start_y,
                                            font_size=xsect_desc_font_size)
            
            xsect_top_port.add_child_object(xsect_top_title_text)


            xsect_top_minval_text \
                = scicomp.TextObject((dataset_title + "_desc_minval"),
                                     xsect_top_minval_string)

            xsect_top_minval_text.set_anchor(x=xsect_desc_start_x,
                                             y=xsect_desc_minval_start_y,
                                             font_size=xsect_desc_font_size)
            
            xsect_top_port.add_child_object(xsect_top_minval_text)


            xsect_top_maxval_text \
                = scicomp.TextObject((dataset_title + "_desc_maxval"),
                                     xsect_top_maxval_string)

            xsect_top_maxval_text.set_anchor(x=xsect_desc_start_x,
                                             y=xsect_desc_maxval_start_y,
                                             font_size=xsect_desc_font_size)
            
            xsect_top_port.add_child_object(xsect_top_maxval_text)


            # Next dataset ...

            dataset_area_start_y -= dataset_port_spacing

        return canvas.get_svg_code(canvas_width)
