// C++ header file mpi_utils.hpp

/*
  By Nathan C. Hearn
     February 10, 2008

  MPI-related utilities.
*/


#ifndef MPI_UTILS_HPP
#define MPI_UTILS_HPP

#ifdef USE_MPI


#include <mpi.h>
#include <vector>


void send_vect(const int dest_rank, const int mpi_tag,
	       const std::vector<unsigned int> & data,
	       const MPI_Comm mpi_comm=MPI_COMM_WORLD);


void recv_vect(const int src_rank, const int mpi_tag,
	       std::vector<unsigned int> & data,
	       const MPI_Comm mpi_comm=MPI_COMM_WORLD);


void send_vect(const int dest_rank, const int mpi_tag,
	       const std::vector<double> & data,
	       const MPI_Comm mpi_comm=MPI_COMM_WORLD);


void recv_vect(const int src_rank, const int mpi_tag,
	       std::vector<double> & data,
	       const MPI_Comm mpi_comm=MPI_COMM_WORLD);


void bcast_vect_server(const std::vector<unsigned int> & src_data,
		       const MPI_Comm mpi_comm=MPI_COMM_WORLD);


void bcast_vect_client(const int src_rank,
		       std::vector<unsigned int> & recv_data, 
		       const MPI_Comm mpi_comm=MPI_COMM_WORLD);


void bcast_vect_server(const std::vector<double> & src_data,
		       const MPI_Comm mpi_comm=MPI_COMM_WORLD);


void bcast_vect_client(const int src_rank, std::vector<double> & recv_data, 
		       const MPI_Comm mpi_comm=MPI_COMM_WORLD);


void sum_vect_master(const std::vector<unsigned int> & local_data,
		     std::vector<unsigned int> & global_sum_data,
		     const MPI_Comm mpi_comm=MPI_COMM_WORLD);


void sum_vect_worker(const int dest_rank, 
		     const std::vector<unsigned int> & local_data,
		     const MPI_Comm mpi_comm=MPI_COMM_WORLD);


void sum_vect_master(const std::vector<double> & local_data,
		     std::vector<double> & global_sum_data,
		     const MPI_Comm mpi_comm=MPI_COMM_WORLD);


void sum_vect_worker(const int dest_rank, 
		     const std::vector<double> & local_data,
		     const MPI_Comm mpi_comm=MPI_COMM_WORLD);


#endif  // USE_MPI

#endif  // MPI_UTILS_HPP
