// C++ header file ranged_bin_arrays.hpp

/*
  By Nathan C. Hearn
     September 15, 2008

  Class for sorting data into separate BinArray objects based on an input 
  parameter.
*/


#ifndef RANGED_BIN_ARRAYS_HPP
#define RANGED_BIN_ARRAYS_HPP


#include <vector>
#include "quickflash_stats_bins.hpp"


// Class RangedBinArrays

class RangedBinArrays
  {
  public :

    RangedBinArrays();

    RangedBinArrays(const double min_range_value, 
		    const std::vector<double> & max_range_values,
		    const double bin_array_min_bounds, 
		    const unsigned int num_bins,
		    const double bin_width);

    RangedBinArrays(const RangedBinArrays & source);

    ~RangedBinArrays() { }

    RangedBinArrays & operator=(const RangedBinArrays & source)
      {
      reset(source);
      return *this;
      }

    const QuickFlash::Stats::BinArray & operator[](const unsigned int index)
      const
      { return get_bin_array(index); }

    void reset();

    void reset(const double min_range_value, 
	       const std::vector<double> & max_range_values,
	       const double bin_array_min_bounds, const unsigned int num_bins, 
	       const double bin_width);

    void reset(const RangedBinArrays & source);

    const QuickFlash::Stats::BinArray & get_bin_array(const unsigned int index)
      const
      { return bin_array_list[index]; }

    unsigned int get_num_bin_arrays() const { return num_bin_arrays; }

    double get_bin_array_range_min() const { return range_min_value; }

    double get_bin_array_range_max(const unsigned int index) const
      { return range_max_values[index]; }

    double get_bin_array_min_bounds() const { return array_min_bounds; }
    double get_bin_array_max_bounds() const { return array_max_bounds; }

    bool pos_in_bin_array(const double bin_array_pos, const double width=0.0)
      const;

    void add_data(const double range_value, const double data_coordinate,
		  const double data_value, const double weight=1.0);

    void add_data_spread(const double range_value, 
			 const double data_coordinate,
			 const double data_value, 
			 const double coordinate_spread,
			 const double weight=1.0);

  private :

    bool locate_bin_array(const double range_value, 
			  unsigned int & bin_array_index) const;

  private :

    unsigned int num_bin_arrays;

    double array_min_bounds;
    double array_max_bounds;

    double range_min_value;
    std::vector<double> range_max_values;

    std::vector<QuickFlash::Stats::BinArray> bin_array_list;
  };


#endif  // RANGED_BIN_ARRAYS_HPP
