// C++ header file structure.hpp

/*
  By Nathan C. Hearn
     February 1, 2008

  Functions for computing structure functions.

  For right now, it is assumed that periodic boundary conditions exist.
*/


#ifndef STRUCTURE_HPP
#define STRUCTURE_HPP

#include <vector>
#include "quickflash_block_blockinfo.hpp"
#include "quickflash_block_blockdata.hpp"
#include "quickflash_stats_bins.hpp"

#ifdef USE_MPI
#include <mpi.h>
#endif  // USE_MPI


typedef std::vector<const QuickFlash::Block::BlockData<double> *> GridVector;


void get_scalar_struct_fn(const QuickFlash::Block::BlockInfo & grid_info,
			  const GridVector & vel_data, 
			  const double bin_width, 
			  const double max_radius, 
			  const std::vector<unsigned int> & fn_order_list,
			  std::vector<double> & r_bin_radii,
			  std::vector<unsigned int> & r_bin_cell_count,
			  std::vector< std::vector<double> > 
			    & fn_values_r_list,
			  std::vector< std::vector<double> > 
			    & fn_stddev_r_list,
			  std::vector<double> & y_bin_positions,
			  std::vector<unsigned int> & y_bin_cell_count,
			  std::vector< std::vector<double> > 
			    & fn_values_y_list,
			  std::vector< std::vector<double> >
			    & fn_stddev_y_list,
			  std::vector<double> & z_bin_positions,
			  std::vector<unsigned int> & z_bin_cell_count,
			  std::vector< std::vector<double> > 
			    & fn_values_z_list,
			  std::vector< std::vector<double> >
			    & fn_stddev_z_list);


void get_scalar_struct_fn(const QuickFlash::Block::BlockInfo & grid_info,
			  const GridVector & vel_data, const double bin_width, 
			  const double max_radius, 
			  const std::vector<unsigned int> & fn_order_list,
			  const unsigned int start_proc_cell_index,
			  const unsigned int num_proc_cells,
			  std::vector<QuickFlash::Stats::BinArray>
			    & function_bins_r_list,
			  std::vector<QuickFlash::Stats::BinArray>
			    & function_bins_y_list,
			  std::vector<QuickFlash::Stats::BinArray>
			    & function_bins_z_list);


#ifdef USE_MPI

extern const int mpi_tag_start_proc_cell_index;
extern const int mpi_tag_fn_order;
extern const int mpi_tag_num_proc_cells;
extern const int mpi_tag_bin_width;
extern const int mpi_tag_max_radius;
extern const int mpi_tag_grid_dims;
extern const int mpi_tag_min_bounds;
extern const int mpi_tag_max_bounds;
extern const int mpi_tag_vel_data;
extern const int mpi_tag_value_weights;
extern const int mpi_tag_weights;


void collect_stats_master(const QuickFlash::Stats::BinArray & bin_array,
			  std::vector<unsigned int> & bin_data_count,
			  std::vector<double> & bin_means,
			  std::vector<double> & bin_stddev,
			  const MPI_Comm mpi_comm=MPI_COMM_WORLD);


void collect_stats_worker(const QuickFlash::Stats::BinArray & bin_array,
			  const MPI_Comm mpi_comm=MPI_COMM_WORLD);


void collect_stats_worker_empty(const MPI_Comm mpi_comm=MPI_COMM_WORLD);


void get_scalar_struct_fn_master(const QuickFlash::Block::BlockInfo & grid_info,
				 const GridVector & vel_data, 
				 const double bin_width, 
				 const double max_radius, 
				 const std::vector<unsigned int> 
				   & fn_order_list,
				 std::vector<double> & r_bin_radii,
				 std::vector<unsigned int> & r_bins_cell_count,
				 std::vector< std::vector<double> > 
				   & fn_values_r_list,
				 std::vector< std::vector<double> > 
				   & fn_stddev_r_list,
				 std::vector<double> & y_bin_positions,
				 std::vector<unsigned int> & y_bins_cell_count,
				 std::vector< std::vector<double> > 
				   & fn_values_y_list,
				 std::vector< std::vector<double> >
				   & fn_stddev_y_list,
				 std::vector<double> & z_bin_positions,
				 std::vector<unsigned int> & z_bins_cell_count,
				 std::vector< std::vector<double> > 
				   & fn_values_z_list,
				 std::vector< std::vector<double> >
				   & fn_stddev_z_list,
				 const MPI_Comm mpi_comm=MPI_COMM_WORLD);


void get_scalar_struct_fn_worker(const MPI_Comm mpi_comm=MPI_COMM_WORLD);


#endif  // USE_MPI


#endif  // STRUCTURE_HPP
