// C++ header file fft.hpp

/*
  By Nathan C. Hearn
     October 30, 2007

  Interface to GSL FFT routines.
*/


#ifndef FFT_HPP
#define FFT_HPP


#include <vector>
#include <complex>
#include <gsl/gsl_fft_complex.h>


int get_power(const std::vector< std::complex<double> > & fft_values,
	      std::vector<double> & spectral_power);

unsigned int get_power_vector_size(const unsigned int num_fft_values);


class FFT
  {
  public :

    FFT();
    ~FFT();

    void reset() const;
    void reset(const unsigned int real_vector_size) const;

    int compute_fft(const std::vector<double> & data_vector,
		    std::vector< std::complex<double> > & trans_vector) const;

  private :

    mutable unsigned int num_elems;

    mutable gsl_fft_complex_workspace * workspace;
    mutable gsl_fft_complex_wavetable * wavetable;
  };


#endif  // FFT_HPP
