// C++ header file line_domain.hpp

/*
  By Nathan C. Hearn
     January 26, 2007

  Class for defining rectangular lineout region parallel to x-axis.
*/


#ifndef LINE_DOMAIN_HPP
#define LINE_DOMAIN_HPP


#include <vector>


class LineDomain
  {
  public:
    LineDomain() : dims(0), start_coords(), end_coords(), radius(0.0) { }
    LineDomain(const double min_x, const double max_x,
	       const std::vector<double> & center_coords,
	       const double domain_radius) :
      dims(0), start_coords(), end_coords(), radius(0.0) 
      { reset(min_x, max_x, center_coords, domain_radius); }
    LineDomain(const LineDomain & source) :
      dims(0), start_coords(), end_coords(), radius(0.0)
      { reset(source); }
    ~LineDomain() { }

    LineDomain & operator=(const LineDomain & source)
      {
      reset(source);
      return *this;
      }

    void reset()
      {
      dims = 0;

      start_coords.clear();
      end_coords.clear();

      radius = 0.0;
      }

    void reset(const double min_x, const double max_x,
	       const std::vector<double> & center_coords,
	       const double domain_radius);

    void reset(const LineDomain & source)
      {
      if (&source != this)
	{
	dims = source.dims;

	start_coords = source.start_coords;
	end_coords = source.end_coords;

	radius = source.radius;
	}
      }

    bool in_domain(const std::vector<double> & point) const;

    bool in_domain(const std::vector<double> & point, 
		   const std::vector<double> & width) const;

    bool in_domain_box(const std::vector<double> & min_coords,
		       const std::vector<double> & max_coords) const;

    
  private:
    unsigned int dims;

    std::vector<double> start_coords;
    std::vector<double> end_coords;

    double radius;
  };


#endif  // LINE_DOMAIN_HPP
