// C++ header file accum_grid.hpp

/*
  By Nathan C. Hearn
     July 6, 2007

  Uniform Grid for accumulating values.
*/


#ifndef ACCUM_GRID_HPP
#define ACCUM_GRID_HPP


#include <vector>
#include "quickflash_block_blockdata.hpp"
#include "quickflash_block_blockinfo.hpp"


// Helper functions

double volume_overlap(const std::vector<double> & primary_min_coords,
		      const std::vector<double> & primary_max_coords,
		      const std::vector<double> & secondary_min_coords,
		      const std::vector<double> & secondary_max_coords);
// Class AccumGrid

class AccumGrid
  {
  public :
    AccumGrid() : dims(0), info(), data() { }

    AccumGrid(const std::vector<double> & min_coords,
	      const std::vector<double> & max_coords,
	      const std::vector<unsigned int> & grid_dims,
	      const std::vector<double> & bin_window_size) :
      dims(0), info(), data(), half_window_width(), bin_window_scale(0.0)
      { reset(min_coords, max_coords, grid_dims, bin_window_size); }

    AccumGrid(const AccumGrid & source) :
      dims(0), info(), data(), half_window_width(), bin_window_scale(0.0)
      { reset(source); }

    ~AccumGrid() { }

    AccumGrid & operator=(const AccumGrid & source)
      {
      reset(source);
      return *this;
      }

    double operator[](const unsigned int index) const 
      { return data[index]; }

    double operator[](const std::vector<unsigned int> & index) const
      { return data[index]; }

    void reset();

    void reset(const std::vector<double> & min_coords,
	       const std::vector<double> & max_coords,
	       const std::vector<unsigned int> & grid_dims,
	       const std::vector<double> & bin_window_size);

    void reset(const AccumGrid & source);

    void clear_grid();

    const std::vector<unsigned int> & get_grid_dims() const 
      { return info.get_block_dims(); }

    double get_cell_volume() const { return info.get_cell_volume(); }

    void distribute_value(const std::vector<double> & center_pos,
			  const double value);

    void distribute_value(const std::vector<double> & center_pos,
			  const std::vector<double> & width,
			  const double value);

  private :

    unsigned int dims;

    QuickFlash::Block::BlockInfo info;
    QuickFlash::Block::BlockData<double> data;

    std::vector<double> half_window_width;
    double bin_window_scale;
  };


#endif  // ACCUM_GRID_HPP
