# composite.py

# By Nathan C. Hearn
#    April 14, 2008


import SciCompositor as scicomp


class SliceComposite :

    def __init__(self, main_title, time, dims, min_envelope_pos,
                 max_envelope_pos, xy_slice_min_coords, xy_slice_max_coords,
                 xz_slice_min_coords=None, xz_slice_max_coords=None,
                 xsect_lower_pos=None, xsect_middle_pos=None,
                 xsect_upper_pos=None, xsect_min_coords=None,
                 xsect_max_coords=None) :

        self.__main_title = main_title
        self.__time = time

        self.__dims = dims
        
        self.__min_envelope_pos = min_envelope_pos
        self.__max_envelope_pos = max_envelope_pos

        self.__xy_slice_min_x = xy_slice_min_coords[0]
        self.__xy_slice_max_x = xy_slice_max_coords[0]

        self.__xy_slice_min_y = xy_slice_min_coords[1]
        self.__xy_slice_max_y = xy_slice_max_coords[1]

        self.__xz_slice_min_x = 0.0
        self.__xz_slice_max_x = 0.0

        self.__xz_slice_min_y = 0.0
        self.__xz_slice_max_y = 0.0

        self.__show_xz_slice = False

        if (dims > 2) :

            if ((xz_slice_min_coords is not None)
                and (xz_slice_max_coords is not None)) :
            
                self.__xz_slice_min_x = xz_slice_min_coords[0]
                self.__xz_slice_min_y = xz_slice_min_coords[1]

                self.__xz_slice_max_x = xz_slice_max_coords[0]
                self.__xz_slice_max_y = xz_slice_max_coords[1]

                self.__show_xz_slice = True

        self.__xy_slice_width = self.__xy_slice_max_x - self.__xy_slice_min_x
        self.__xy_slice_height = self.__xy_slice_max_y - self.__xy_slice_min_y

        self.__xy_slice_aspect_h_w \
            = self.__xy_slice_height / self.__xy_slice_width

        self.__xz_slice_width = 0.0
        self.__xz_slice_height = 0.0

        self.__xz_slice_aspect_h_w = 0.0
        
        if (self.__show_xz_slice) :
            
            self.__xz_slice_width \
                = self.__xz_slice_max_x - self.__xz_slice_min_x
            
            self.__xz_slice_height \
                = self.__xz_slice_max_y - self.__xz_slice_min_y

            self.__xz_slice_aspect_h_w \
                = self.__xz_slice_height / self.__xz_slice_width

        self.__show_xsect_slice = False

        self.__xsect_min_y = 0.0
        self.__xsect_max_y = 0.0

        self.__xsect_min_z = 0.0
        self.__xsect_max_z = 0.0

        self.__xsect_width = 0.0
        self.__xsect_height = 0.0
        self.__xsect_aspect_h_w = 0.0

        if (dims > 2) :

            if ((xsect_min_coords is not None)
                and (xsect_max_coords is not None)) :

                self.__xsect_min_y = xsect_min_coords[0]
                self.__xsect_max_y = xsect_max_coords[0]

                self.__xsect_min_z = xsect_min_coords[1]
                self.__xsect_max_z = xsect_max_coords[1]

                self.__xsect_width = self.__xsect_max_y - self.__xsect_min_y
                self.__xsect_height = self.__xsect_max_z - self.__xsect_min_z

                self.__xsect_aspect_h_w \
                    = self.__xsect_height / self.__xsect_width

                self.__show_xsect_slice = True

        self.__xsect_lower_pos = xsect_lower_pos
        self.__xsect_middle_pos = xsect_middle_pos
        self.__xsect_upper_pos = xsect_upper_pos

        self.__dataset_titles = list()
        self.__dataset_min_values = list()
        self.__dataset_max_values = list()

        self.__xy_slice_image_files = list()
        self.__xy_slice_min_values = list()
        self.__xy_slice_max_values = list()
        
        self.__xz_slice_image_files = None
        self.__xz_slice_min_values = None
        self.__xz_slice_max_values = None

        if (self.__show_xz_slice) :
            self.__xz_slice_image_files = list()
            self.__xz_slice_min_values = list()
            self.__xz_slice_max_values = list()
           
        self.__lower_xsect_image_files = list()
        self.__lower_xsect_min_values = list()
        self.__lower_xsect_max_values = list()

        self.__middle_xsect_image_files = None
        self.__middle_xsect_min_values = None
        self.__middle_xsect_max_values = None

        self.__upper_xsect_image_files = None
        self.__upper_xsect_min_values = None
        self.__upper_xsect_max_values = None

        if (self.__show_xsect_slice) :
            
            self.__lower_xsect_image_files = list()
            self.__lower_xsect_min_values = list()
            self.__lower_xsect_max_values = list()

            self.__middle_xsect_image_files = list()
            self.__middle_xsect_min_values = list()
            self.__middle_xsect_max_values = list()

            self.__upper_xsect_image_files = list()
            self.__upper_xsect_min_values = list()
            self.__upper_xsect_max_values = list()


    def add_dataset(self, dataset_title, dataset_min_value, dataset_max_value,
                    xy_image_file, xy_slice_min_value, xy_slice_max_value,
                    xz_image_file, xz_slice_min_value,
                    xz_slice_max_value,
                    lower_xsect_image_file, 
                    lower_xsect_min_value, lower_xsect_max_value,
                    middle_front_xsect_image_file, 
                    middle_front_xsect_min_value, middle_xsect_max_value,
                    upper_xsect_image_file, 
                    upper_xsect_min_value, upper_xsect_max_value) :

        self.__dataset_titles.append(dataset_title)

        self.__dataset_min_values.append(dataset_min_value)
        self.__dataset_max_values.append(dataset_max_value)

        self.__xy_slice_image_files.append(xy_image_file)

        self.__xy_slice_min_values.append(xy_slice_min_value)
        self.__xy_slice_max_values.append(xy_slice_max_value)

        if (self.__show_xz_slice) :

            self.__xz_slice_image_files.append(xz_image_file)

            self.__xz_slice_min_values.append(xz_slice_min_value)
            self.__xz_slice_max_values.append(xz_slice_max_value)

        if (self.__show_xsect_slice) :
            
            self.__lower_xsect_image_files.append(lower_xsect_image_file)
            self.__lower_xsect_min_values.append(lower_xsect_min_value)
            self.__lower_xsect_max_values.append(lower_xsect_max_value)

            self.__middle_xsect_image_files.append(middle_xsect_image_file)
            self.__middle_xsect_min_values.append(middle_xsect_min_value)
            self.__middle_xsect_max_values.append(middle_xsect_max_value)

            self.__upper_xsect_image_files.append(upper_xsect_image_file)
            self.__upper_xsect_min_values.append(upper_xsect_min_value)
            self.__upper_xsect_max_values.append(upper_xsect_max_value)


    def generate_svg(self, canvas_width) :

        from SciCompositor import svg_defs

        num_datasets = len(self.__dataset_titles)

        slice_port_image_width = 0.85 * canvas_width

        dataset_port_spacing_factor = 1.3
        
        xy_slice_port_image_height \
            = self.__xy_slice_aspect_h_w * slice_port_image_width

        xz_slice_port_image_height = 0.0

        if (self.__show_xz_slice) :
            
            xz_slice_port_image_height \
                = self.__xz_slice_aspect_h_w * slice_port_image_width

            dataset_port_spacing_factor += 1.3

        slice_port_image_height = max(xy_slice_port_image_height,
                                      xz_slice_port_image_height)

        xsect_port_height = 0.0
        xsect_port_width = 0.0

        if (self.__show_xsect_slice) :
            
            xsect_port_height = slice_port_image_height
        
            xsect_port_width = xsect_port_height / self.__xsect_aspect_h_w

            dataset_port_spacing_factor += 1.3

        dataset_port_spacing \
            = dataset_port_spacing_factor * slice_port_image_height

        canvas_height = (float(num_datasets) + 0.2) * dataset_port_spacing

        header_start_x = 0.01 * canvas_width
        header_start_y = canvas_height - (0.15 * dataset_port_spacing)

        header_end_x = 0.9 * canvas_width
        header_end_y = header_start_y + (0.125 * dataset_port_spacing)

        header_width = header_end_x - header_start_x
        header_height = header_end_y - header_start_y

        main_title_font_size = 0.3 * header_height
        
        dataset_title_start_x = 0.01 * canvas_width
        dataset_title_start_y = 0.95 * dataset_port_spacing

        dataset_area_start_x = 0.0
        dataset_area_start_y = canvas_height - (1.15 * dataset_port_spacing)

        dataset_title_font_size = 0.04 * dataset_port_spacing
        dataset_desc_font_size = 0.03 * dataset_port_spacing

        dataset_desc_start_x = 0.9 * canvas_width
        dataset_desc_start_y = dataset_title_start_y

        dataset_desc_text_anchor = svg_defs.svg_text_anchor_end

        slice_desc_font_size = 0.03 * dataset_port_spacing

        slice_port_desc_start_x = 0.0 * canvas_width
        slice_port_desc_start_y = -0.12 * slice_port_image_height

        xy_slice_port_image_start_x = 0.1 * canvas_width
        xy_slice_port_image_start_y = dataset_port_spacing \
                                      - (1.1 * slice_port_image_height)

        xy_slice_port_image_end_x = xy_slice_port_image_start_x \
                                    + slice_port_image_width
        
        xy_slice_port_image_end_y = xy_slice_port_image_start_y \
                                    + slice_port_image_height

        xz_slice_port_image_start_x = 0.0
        xz_slice_port_image_start_y = 0.0

        xz_slice_port_image_end_x = 0.0
        xz_slice_port_image_end_y = 0.0

        if (self.__show_xz_slice) :
            
            xz_slice_port_image_start_x = 0.1 * canvas_width
            xz_slice_port_image_start_y = xy_slice_port_image_start_y \
                                          - (1.1 * slice_port_image_height)

            xz_slice_port_image_end_x = xz_slice_port_image_start_x \
                                        + slice_port_image_width
        
            xz_slice_port_image_end_y = xz_slice_port_image_start_y \
                                        + slice_port_image_height

        slice_axes_font_size = 0.05 * self.__xy_slice_height

        slice_axes_horiz_ticks = 10
        slice_axes_vert_ticks = 5

        xsect_axes_horiz_ticks = 1
        xsect_axes_vert_ticks = 5

        xsect_port_start_y = 0.0
        xsect_port_end_y = 0.0

        xsect_desc_start_x = 0.0

        xsect_desc_title_start_y = 0.0
        xsect_desc_minval_start_y = 0.0
        xsect_desc_maxval_start_y = 0.0

        xsect_desc_font_size = 0.0

        xsect_lower_port_start_x = 0.0
        xsect_lower_port_end_x = 0.0

        xsect_middle_port_start_x = 0.0
        xsect_middle_port_end_x = 0.0

        xsect_top_port_start_x = 0.0
        xsect_top_port_end_x = 0.0

        if (self.__show_xsect_slice) :

            xsect_port_start_y = xy_slice_port_image_start_y

            if (self.__show_xz_slice) :
                xsect_port_start_y = xz_slice_port_image_start_y

            xsect_port_start_y -= 1.1 * slice_port_image_height

            xsect_port_start_y = 0.1 * dataset_port_spacing
            xsect_port_end_y = xsect_port_start_y + xsect_port_height

            xsect_desc_start_x = 1.1 * xsect_port_width

            xsect_desc_title_start_y = 0.8 * xsect_port_height
            xsect_desc_minval_start_y = 0.5 * xsect_port_height
            xsect_desc_maxval_start_y = 0.2 * xsect_port_height

            xsect_desc_font_size = 0.75 * dataset_desc_font_size
        
            xsect_lower_port_start_x = 0.1 * canvas_width
            xsect_lower_port_end_x = xsect_lower_port_start_x \
                                     + xsect_port_width
        
            xsect_middle_port_start_x = 0.43 * canvas_width
            xsect_middle_port_end_x = xsect_middle_port_start_x \
                                      + xsect_port_width
        
            xsect_upper_port_start_x = 0.76 * canvas_width
            xsect_upper_port_end_x = xsect_upper_port_start_x \
                                     + xsect_port_width
        
        # Create the canvas

        canvas = scicomp.Canvas(0.0, canvas_width, 0.0, canvas_height)

        # Build the header

        header_port = canvas.get_new_viewport("header_port",
                                              header_start_x, header_end_x,
                                              header_start_y, header_end_y,
                                              0.0, header_width,
                                              0.0, header_height)

        header_text_string = self.__main_title \
                             + " - Time " + str(self.__time)

        header_subtext_string = "Envelope x-range " \
                                + str(self.__min_envelope_pos) + " - " \
                                + str(self.__max_envelope_pos)

        header_text = scicomp.TextObject("header_text", header_text_string)

        header_text.set_anchor(x=0.0, y=(0.6 * header_height),
                               font_size=main_title_font_size)

        header_port.add_child_object(header_text)

        header_subtext = scicomp.TextObject("header_subtext",
                                            header_subtext_string)

        header_subtext.set_anchor(x=0.0, y=(0.1 * header_height),
                                  font_size=main_title_font_size)

        header_port.add_child_object(header_subtext)

        # Run through the datasets

        for set_index in range(num_datasets) :

            dataset_title = self.__dataset_titles[set_index]

            dataset_min_value = self.__dataset_min_values[set_index]
            dataset_max_value = self.__dataset_max_values[set_index]

            xy_slice_image_file = self.__xy_slice_image_files[set_index]
            
            xy_slice_min_value = self.__xy_slice_min_values[set_index]
            xy_slice_max_value = self.__xy_slice_max_values[set_index]

            xz_slice_image_file = None

            xz_slice_min_value = 0.0
            xz_slice_max_value = 0.0

            if (self.__show_xz_slice) :
                
                xz_slice_image_file = self.__xz_slice_image_files[set_index]
            
                xz_slice_min_value = self.__xz_slice_min_values[set_index]
                xz_slice_max_value = self.__xz_slice_max_values[set_index]

            xsect_lower_image_file = None

            xsect_lower_min_value = 0.0
            xsect_lower_max_value = 0.0

            xsect_middle_image_file = None

            xsect_middle_min_value = 0.0
            xsect_middle_max_value = 0.0

            xsect_upper_image_file = None

            xsect_upper_min_value = 0.0
            xsect_upper_max_value = 0.0

            if (self.__show_xsect_slice) :
                
                xsect_lower_image_file \
                     = self.__lower_xsect_image_files[set_index]

                xsect_lower_min_value \
                  = self.__lower_xsect_min_values[set_index]
                
                xsect_lower_max_value \
                     = self.__lower_xsect_max_values[set_index]

                xsect_middle_image_file \
                     = self.__middle_xsect_image_files[set_index]

                xsect_middle_min_value \
                     = self.__middle_xsect_min_values[set_index]
                
                xsect_middle_max_value \
                     = self.__middle_xsect_max_values[set_index]

                xsect_upper_image_file \
                     = self.__upper_xsect_image_files[set_index]

                xsect_upper_min_value \
                     = self.__upper_xsect_min_values[set_index]
                
                xsect_upper_max_value \
                     = self.__upper_xsect_max_values[set_index]

            # Set up the port for slice information and plot

            dataset_port_min_x = 0.0
            dataset_port_max_x = canvas_width

            dataset_port_min_y = dataset_area_start_y
            dataset_port_max_y = dataset_area_start_y + dataset_port_spacing

            dataset_port \
                = canvas.get_new_viewport((dataset_title + "_port"),
                                          dataset_port_min_x,
                                          dataset_port_max_x,
                                          dataset_port_min_y,
                                          dataset_port_max_y,
                                          0.0, canvas_width,
                                          0.0, dataset_port_spacing)

            # Write the title

            title_text_string = "Dataset " + dataset_title

            title_text = scicomp.TextObject((dataset_title + "_title"),
                                            title_text_string)

            title_text.set_anchor(x=dataset_title_start_x,
                                  y=dataset_title_start_y,
                                  font_size=dataset_title_font_size)

            dataset_port.add_child_object(title_text)

            # Write the descripton

            dataset_desc_text_string = "Dataset Min Value [ " \
                                       + str(dataset_min_value) \
                                       + " ] Max Value [ " \
                                       + str(dataset_max_value) \
                                       + " ]"

            dataset_desc_text = scicomp.TextObject((dataset_title + "_desc"),
                                                   dataset_desc_text_string)

            dataset_desc_text.set_anchor(x=dataset_desc_start_x,
                                         y=dataset_desc_start_y,
                                         font_size=dataset_desc_font_size,
                                         anchor_pos=dataset_desc_text_anchor)
            
            dataset_port.add_child_object(dataset_desc_text)

            # Build the xy Plot port

            xy_plot_port \
                = dataset_port.get_sub_viewport((dataset_title + "_xy_plot"),
                                                xy_slice_port_image_start_x,
                                                xy_slice_port_image_end_x,
                                                xy_slice_port_image_start_y,
                                                xy_slice_port_image_end_y,
                                                0.0, slice_port_image_width,
                                                0.0, slice_port_image_height)

            xy_plot_port.set_physical_coord_map(self.__xy_slice_min_x,
                                                self.__xy_slice_max_x,
                                                self.__xy_slice_min_y,
                                                self.__xy_slice_max_y)

            xy_plot_img = scicomp.ImageObject((dataset_title + "_xy_img"),
                                              xy_slice_image_file,
                                              self.__xy_slice_min_x,
                                              self.__xy_slice_max_x,
                                              self.__xy_slice_min_y,
                                              self.__xy_slice_max_y,
                                              use_physical_coords=True)

            xy_plot_port.add_child_object(xy_plot_img)

            xy_plot_axes = scicomp.AxesObject((dataset_title + "_xy_axes"),
                                              self.__xy_slice_min_x,
                                              self.__xy_slice_max_x,
                                              self.__xy_slice_min_y,
                                              self.__xy_slice_max_y,
                                              use_physical_coords=True)

            xy_plot_axes.set_frame_color(red=1.0, green=0.0, blue=0.0)
            xy_plot_axes.set_frame_line_width(1.0)

            horiz_ticks = slice_axes_horiz_ticks
            vert_ticks = slice_axes_vert_ticks
            
            xy_plot_axes.set_x_major_ticks_regular(max_ticks=horiz_ticks)
            xy_plot_axes.set_y_major_ticks_regular(max_ticks=vert_ticks)

            xy_plot_axes.set_major_ticks_color(red=0.7, green=0.0, blue=0.7,
                                               opacity=0.5)
            xy_plot_axes.set_major_ticks_line_width(1.0)

            xy_plot_axes.set_axis_label_font_size(slice_axes_font_size)

            xy_plot_port.add_child_object(xy_plot_axes)

            # Write the descripton

            xy_desc_text_string = "x-y Plane - Slice Min Value [ " \
                                  + str(xy_slice_min_value) \
                                  + " ] Max Value [ " \
                                  + str(xy_slice_max_value) \
                                  + " ]"

            xy_desc_text = scicomp.TextObject((dataset_title + "_desc"),
                                              xy_desc_text_string)

            xy_desc_text.set_anchor(x=slice_port_desc_start_x,
                                    y=slice_port_desc_start_y,
                                    font_size=dataset_desc_font_size)
            
            xy_plot_port.add_child_object(xy_desc_text)

            # Build the xz Plot port

            if (self.__show_xz_slice) :
                
                xz_plot_port \
                    = dataset_port.get_sub_viewport((dataset_title
                                                     + "_xz_plot"),
                                                    xz_slice_port_image_start_x,
                                                    xz_slice_port_image_end_x,
                                                    xz_slice_port_image_start_y,
                                                    xz_slice_port_image_end_y,
                                                    0.0, slice_port_image_width,
                                                    0.0, slice_port_image_height)

                xz_plot_port.set_physical_coord_map(self.__xz_slice_min_x,
                                                    self.__xz_slice_max_x,
                                                    self.__xz_slice_min_y,
                                                    self.__xz_slice_max_y)

                xz_plot_img = scicomp.ImageObject((dataset_title + "_xz_img"),
                                                  xz_slice_image_file,
                                                  self.__xz_slice_min_x,
                                                  self.__xz_slice_max_x,
                                                  self.__xz_slice_min_y,
                                                  self.__xz_slice_max_y,
                                                  use_physical_coords=True)

                xz_plot_port.add_child_object(xz_plot_img)

                xz_plot_axes = scicomp.AxesObject((dataset_title + "_xz_axes"),
                                                  self.__xz_slice_min_x,
                                                  self.__xz_slice_max_x,
                                                  self.__xz_slice_min_y,
                                                  self.__xz_slice_max_y,
                                                  use_physical_coords=True)

                xz_plot_axes.set_frame_color(red=1.0, green=0.0, blue=0.0)
                xz_plot_axes.set_frame_line_width(1.0)

                horiz_ticks = slice_axes_horiz_ticks
                vert_ticks = slice_axes_vert_ticks
            
                xz_plot_axes.set_x_major_ticks_regular(max_ticks=horiz_ticks)
                xz_plot_axes.set_y_major_ticks_regular(max_ticks=vert_ticks)

                xz_plot_axes.set_major_ticks_color(red=0.7, green=0.0,
                                                   blue=0.7, opacity=0.5)
                xz_plot_axes.set_major_ticks_line_width(1.0)

                xz_plot_axes.set_axis_label_font_size(slice_axes_font_size)

                xz_plot_port.add_child_object(xz_plot_axes)

                # Write the descripton

                xz_desc_text_string = "x-z Plane - Slice Min Value [ " \
                                      + str(xz_slice_min_value) \
                                      + " ] Max Value [ " \
                                      + str(xz_slice_max_value) \
                                      + " ]"

                xz_desc_text = scicomp.TextObject((dataset_title + "_desc"),
                                                  xz_desc_text_string)

                xz_desc_text.set_anchor(x=slice_port_desc_start_x,
                                        y=slice_port_desc_start_y,
                                        font_size=dataset_desc_font_size)
            
                xz_plot_port.add_child_object(xz_desc_text)


            if (self.__show_xsect_slice) :

                # Build the lower cross-section plot port

                xsect_lower_port \
                     = dataset_port.get_sub_viewport((dataset_title
                                                      + "_xsect_lower_plot"),
                                                     xsect_lower_port_start_x,
                                                     xsect_lower_port_end_x,
                                                     xsect_port_start_y,
                                                     xsect_port_end_y,
                                                     0.0, xsect_port_width,
                                                     0.0, xsect_port_height)

                xsect_lower_port.set_physical_coord_map(self.__xsect_min_x,
                                                        self.__xsect_max_x,
                                                        self.__xsect_min_y,
                                                        self.__xsect_max_y)

                xsect_lower_img = scicomp.ImageObject((dataset_title
                                                       + "_xsect_lower_img"),
                                                      xsect_lower_image_file,
                                                      self.__xsect_min_x,
                                                      self.__xsect_max_x,
                                                      self.__xsect_min_y,
                                                      self.__xsect_max_y,
                                                      use_physical_coords=True)

                xsect_lower_port.add_child_object(xsect_lower_img)

                xsect_lower_axes = scicomp.AxesObject((dataset_title
                                                       + "_xsect_lower_axes"),
                                                      self.__xsect_min_x,
                                                      self.__xsect_max_x,
                                                      self.__xsect_min_y,
                                                      self.__xsect_max_y,
                                                      use_physical_coords=True)

                xsect_lower_axes.set_frame_color(red=1.0, green=0.0, blue=0.0)
                xsect_lower_axes.set_frame_line_width(1.0)

                horiz_ticks = xsect_axes_horiz_ticks
                vert_ticks = xsect_axes_vert_ticks
            
                xsect_lower_axes.set_x_major_ticks_regular(max_ticks=horiz_ticks)
                xsect_lower_axes.set_y_major_ticks_regular(max_ticks=vert_ticks)

                xsect_lower_axes.set_major_ticks_color(red=0.7, green=0.0,
                                                       blue=0.7, opacity=0.5)
                
                xsect_lower_axes.set_major_ticks_line_width(1.0)

                xsect_lower_axes.set_axis_label_font_size(slice_axes_font_size)

                xsect_lower_port.add_child_object(xsect_lower_axes)

                # Write the descripton

                xsect_lower_title_string = "y-z Plane at x = " \
                                           + str(self.__xsect_lower_pos)

                xsect_lower_minval_string = "Min Value [ " \
                                            + str(xsect_lower_min_value) + " ]"

                xsect_lower_maxval_string = "Max Value [ " \
                                            + str(xsect_lower_max_value) + " ]"

                xsect_lower_title_text \
                     = scicomp.TextObject((dataset_title + "_desc_title"),
                                          xsect_lower_title_string)

                xsect_lower_title_text.set_anchor(x=xsect_desc_start_x,
                                                  y=xsect_desc_title_start_y,
                                                  font_size=xsect_desc_font_size)
                
                xsect_lower_port.add_child_object(xsect_lower_title_text)


                xsect_lower_minval_text \
                     = scicomp.TextObject((dataset_title + "_desc_minval"),
                                          xsect_lower_minval_string)

                xsect_lower_minval_text.set_anchor(x=xsect_desc_start_x,
                                                   y=xsect_desc_minval_start_y,
                                                   font_size=xsect_desc_font_size)
                
                xsect_lower_port.add_child_object(xsect_lower_minval_text)


                xsect_lower_maxval_text \
                     = scicomp.TextObject((dataset_title + "_desc_maxval"),
                                          xsect_lower_maxval_string)

                xsect_lower_maxval_text.set_anchor(x=xsect_desc_start_x,
                                                   y=xsect_desc_maxval_start_y,
                                                   font_size=xsect_desc_font_size)
            
                xsect_lower_port.add_child_object(xsect_lower_maxval_text)


                # Build the middle-front cross-section plot port

                xsect_middle_port \
                     = dataset_port.get_sub_viewport((dataset_title
                                                      + "_xsect_middle_plot"),
                                                     xsect_middle_port_start_x,
                                                     xsect_middle_port_end_x,
                                                     xsect_port_start_y,
                                                     xsect_port_end_y,
                                                     0.0, xsect_port_width,
                                                     0.0, xsect_port_height)

                xsect_middle_port.set_physical_coord_map(self.__xsect_min_x,
                                                         self.__xsect_max_x,
                                                         self.__xsect_min_y,
                                                         self.__xsect_max_y)

                xsect_middle_img = scicomp.ImageObject((dataset_title
                                                        + "_xsect_middle_img"),
                                                       xsect_middle_image_file,
                                                       self.__xsect_min_x,
                                                       self.__xsect_max_x,
                                                       self.__xsect_min_y,
                                                       self.__xsect_max_y,
                                                       use_physical_coords=True)

                xsect_middle_port.add_child_object(xsect_middle_img)

                xsect_middle_axes = scicomp.AxesObject((dataset_title
                                                        + "_xsect_middle_axes"),
                                                       self.__xsect_min_x,
                                                       self.__xsect_max_x,
                                                       self.__xsect_min_y,
                                                       self.__xsect_max_y,
                                                       use_physical_coords=True)

                xsect_middle_axes.set_frame_color(red=1.0, green=0.0, blue=0.0)
                xsect_middle_axes.set_frame_line_width(1.0)

                horiz_ticks = xsect_axes_horiz_ticks
                vert_ticks = xsect_axes_vert_ticks
            
                xsect_middle_axes.set_x_major_ticks_regular(max_ticks=horiz_ticks)
                xsect_middle_axes.set_y_major_ticks_regular(max_ticks=vert_ticks)

                xsect_middle_axes.set_major_ticks_color(red=0.7, green=0.0,
                                                        blue=0.7, opacity=0.5)
                
                xsect_middle_axes.set_major_ticks_line_width(1.0)

                xsect_middle_axes.set_axis_label_font_size(slice_axes_font_size)

                xsect_middle_port.add_child_object(xsect_middle_axes)

                # Write the descripton

                xsect_middle_title_string = "y-z Plane at x = " \
                                            + str(self.__xsect_middle_pos)

                xsect_middle_minval_string = "Min Value [ " \
                                             + str(xsect_middle_min_value) + " ]"

                xsect_middle_maxval_string = "Max Value [ " \
                                             + str(xsect_middle_max_value) + " ]"
                
                xsect_middle_title_text \
                     = scicomp.TextObject((dataset_title + "_desc_title"),
                                          xsect_middle_title_string)

                xsect_middle_title_text.set_anchor(x=xsect_desc_start_x,
                                                   y=xsect_desc_title_start_y,
                                                   font_size=xsect_desc_font_size)
            
                xsect_middle_port.add_child_object(xsect_middle_title_text)


                xsect_middle_minval_text \
                     = scicomp.TextObject((dataset_title + "_desc_minval"),
                                          xsect_middle_minval_string)

                xsect_middle_minval_text.set_anchor(x=xsect_desc_start_x,
                                                    y=xsect_desc_minval_start_y,
                                                    font_size=xsect_desc_font_size)
            
                xsect_middle_port.add_child_object(xsect_middle_minval_text)


                xsect_middle_maxval_text \
                     = scicomp.TextObject((dataset_title + "_desc_maxval"),
                                          xsect_middle_maxval_string)

                xsect_middle_maxval_text.set_anchor(x=xsect_desc_start_x,
                                                    y=xsect_desc_maxval_start_y,
                                                    font_size=xsect_desc_font_size)
            
                xsect_middle_port.add_child_object(xsect_middle_maxval_text)


                # Build the upper cross-section plot port

                xsect_upper_port \
                     = dataset_port.get_sub_viewport((dataset_title
                                                      + "_xsect_upper_plot"),
                                                     xsect_upper_port_start_x,
                                                     xsect_upper_port_end_x,
                                                     xsect_port_start_y,
                                                     xsect_port_end_y,
                                                     0.0, xsect_port_width,
                                                     0.0, xsect_port_height)

                xsect_upper_port.set_physical_coord_map(self.__xsect_min_x,
                                                        self.__xsect_max_x,
                                                        self.__xsect_min_y,
                                                        self.__xsect_max_y)

                xsect_upper_img = scicomp.ImageObject((dataset_title
                                                       + "_xsect_upper_img"),
                                                      xsect_upper_image_file,
                                                      self.__xsect_min_x,
                                                      self.__xsect_max_x,
                                                      self.__xsect_min_y,
                                                      self.__xsect_max_y,
                                                      use_physical_coords=True)

                xsect_upper_port.add_child_object(xsect_upper_img)

                xsect_upper_axes = scicomp.AxesObject((dataset_title
                                                       + "_xsect_upper_axes"),
                                                      self.__xsect_min_x,
                                                      self.__xsect_max_x,
                                                      self.__xsect_min_y,
                                                      self.__xsect_max_y,
                                                      use_physical_coords=True)

                xsect_upper_axes.set_frame_color(red=1.0, green=0.0, blue=0.0)
                
                xsect_upper_axes.set_frame_line_width(1.0)

                horiz_ticks = xsect_axes_horiz_ticks
                vert_ticks = xsect_axes_vert_ticks
            
                xsect_upper_axes.set_x_major_ticks_regular(max_ticks=horiz_ticks)
                xsect_upper_axes.set_y_major_ticks_regular(max_ticks=vert_ticks)

                xsect_upper_axes.set_major_ticks_color(red=0.7, green=0.0,
                                                       blue=0.7, opacity=0.5)
                
                xsect_upper_axes.set_major_ticks_line_width(1.0)

                xsect_upper_axes.set_axis_label_font_size(slice_axes_font_size)

                xsect_upper_port.add_child_object(xsect_upper_axes)

                # Write the descripton

                xsect_upper_title_string = "y-z Plane at x = " \
                                           + str(self.__xsect_upper_pos)

                xsect_upper_minval_string = "Min Value [ " \
                                            + str(xsect_upper_min_value) + " ]"

                xsect_upper_maxval_string = "Max Value [ " \
                                            + str(xsect_upper_max_value) + " ]"

                xsect_upper_title_text \
                     = scicomp.TextObject((dataset_title + "_desc_title"),
                                          xsect_upper_title_string)

                xsect_upper_title_text.set_anchor(x=xsect_desc_start_x,
                                                  y=xsect_desc_title_start_y,
                                                  font_size=xsect_desc_font_size)
            
                xsect_upper_port.add_child_object(xsect_upper_title_text)


                xsect_upper_minval_text \
                     = scicomp.TextObject((dataset_title + "_desc_minval"),
                                          xsect_upper_minval_string)

                xsect_upper_minval_text.set_anchor(x=xsect_desc_start_x,
                                                   y=xsect_desc_minval_start_y,
                                                   font_size=xsect_desc_font_size)
            
                xsect_upper_port.add_child_object(xsect_upper_minval_text)


                xsect_upper_maxval_text \
                     = scicomp.TextObject((dataset_title + "_desc_maxval"),
                                          xsect_upper_maxval_string)

                xsect_upper_maxval_text.set_anchor(x=xsect_desc_start_x,
                                                   y=xsect_desc_maxval_start_y,
                                                   font_size=xsect_desc_font_size)
            
                xsect_upper_port.add_child_object(xsect_upper_maxval_text)


            # Next dataset ...

            dataset_area_start_y -= dataset_port_spacing

        return canvas.get_svg_code(canvas_width)
