// C++ program file grid_output.cpp

/*
  By Nathan C. Hearn
     September 25, 2007

  File output routines for SliceGrid objects.
*/


#include "grid_output.hpp"
#include <math.h>
#include <string>
#include <list>
#include <fstream>
#include <iostream>
#include <vector>
#include "quickflash_except.hpp"
#include "quickflash_slice_slicegrid.hpp"



const double rad_to_deg = 180.0 / (M_PI);


void write_grid(const std::string & filename, 
		const QuickFlash::Slice::SliceGrid & data)
  {
  std::list<std::string> empty_comment_list;

  write_grid(filename, data, empty_comment_list);
  }


void write_grid(const std::string & filename, 
		const QuickFlash::Slice::SliceGrid & data,
		const std::list<std::string> & comment_list)
  {
  using std::endl;

  std::ofstream outfile(filename.c_str());

  outfile << "# Grid data output [ " << filename << " ]" << endl;

  std::list<std::string>::const_iterator str_iter = comment_list.begin();
  std::list<std::string>::const_iterator end_str_iter = comment_list.end();

  while (str_iter != end_str_iter)
    {
    outfile << "# " << *str_iter << endl;
    ++str_iter;
    }

  const std::vector<double> & grid_center = data.get_grid_center();

  const unsigned int num_space_dims = grid_center.size();
  
  outfile << "# Grid center [";

  for (unsigned int i = 0; i < num_space_dims; i++)
    outfile << " " << grid_center[i];

  outfile << " ]" << endl;

  const std::vector<unsigned int> & grid_dims = data.get_grid_dims();

  const unsigned int num_grid_dims = grid_dims.size();

  if (num_grid_dims != 2)
    throw QuickFlash::Except("Incompatible grid dimensions");

  outfile << "# Grid pixels [";

  for (unsigned int i = 0; i < 2; i++)
    outfile << " " << grid_dims[i];

  outfile << " ]" << endl;

  const std::vector<double> & grid_width = data.get_slice_grid_width();

  outfile << "# Grid width [";

  for (unsigned int i = 0; i < 2; i++)
    outfile << " " << grid_width[i];

  outfile << " ]" << endl;

  const double roll_angle_deg = data.get_roll_angle() * rad_to_deg;
  const double polar_angle_deg = data.get_polar_angle() * rad_to_deg;
  const double azim_angle_deg = data.get_azimuthal_angle() * rad_to_deg;

  outfile << "# Rotation angles roll [ " << roll_angle_deg << " ] polar [ "
	  << polar_angle_deg << " ] azimuthal [ " << azim_angle_deg 
	  << " ] degrees" << endl;

  double min_value = 0.0;
  double max_value = 0.0;

  data.get_minmax_values(min_value, max_value);

  outfile << "# Data values min [ " << min_value << " ] max [ " << max_value
	  << " ]" << endl;

  // Write out the data

  const unsigned int max_i = grid_dims[0];
  const unsigned int max_j = grid_dims[1];

  std::vector<unsigned int> pixel_index(2);

  for (unsigned int i = 0; i < max_i; i++)
    {
    pixel_index[0] = i;

    for (unsigned int j = 0; j < max_j; j++)
      {
      pixel_index[1] = j;

      outfile << data[pixel_index] << " ";
      }

    outfile << endl;
    }

  outfile << endl;
  }
