// C++ header file shock_front.hpp

/*
  By Nathan C. Hearn
     April 28, 2008

  Routines for locating the shock front in Omega RT simulations.
*/


#ifndef SHOCK_FRONT_HPP
#define SHOCK_FRONT_HPP


#include <vector>
#include <string>
#include "quickflash_file_datafile.hpp"


bool range_radius_overlap(const std::vector<double> & min_range_coords,
			  const std::vector<double> & max_range_coords,
			  const std::vector<double> & radius_center,
			  const double max_radius);


void project_vector(const unsigned int skip_axis_index,
		    const std::vector<double> & full_vector,
		    std::vector<double> & reduced_vector);


int find_axis_peak(const QuickFlash::File::DataFile & df,
		   const std::string & dataset_name, 
		   const unsigned int search_axis, double & peak_coord,
		   const bool limit_sample_radius=false,
		   const double max_sample_radius=0.0,
		   const bool limit_axis_bounds=false,
		   const double min_axis_bound=0.0,
		   const double max_axis_bound=0.0);


int find_axis_peak(const QuickFlash::File::DataFile & df,
		   const char dataset_name[], 
		   const unsigned int search_axis, double & peak_coord,
		   const bool limit_sample_radius=false,
		   const double max_sample_radius=0.0,
		   const bool limit_axis_bounds=false,
		   const double min_axis_bound=0.0,
		   const double max_axis_bound=0.0);


#endif  // SHOCK_FRONT_HPP
