#!/usr/bin/env python

# particle_extract.py

# By Nathan C. Hearn
#    August 2, 2007
#
# Front end to particle extractor


def read_tag_file(filename) :

    tag_list = list()

    f = file(filename, "r")

    end_reached = False

    while (not end_reached) :

        line = f.readline()

        if (line == "") :
               
            end_reached = True

        else :

            work_line = None

            comment_pos = line.find("#")

            if (comment_pos >= 0) :
                work_line = line[0:comment_pos]  # Everything before the "#"
            else :
                work_line = line

            words = work_line.split()

            for num_word in words :
                tag_list.append(int(num_word))

    return tag_list


def main() :

    from sys import exit, stderr, stdout
    from optparse import OptionParser

    usage = str("Usage: %prog file_prefix start_file_index end_file_index "
                + "data_name1 [ data_name2 ... ] [ options ]")
    version = str("%prog 0.1.0")

    argparser = OptionParser(usage, version=version)

    output_prefix_str = str("output-prefix")
    argparser.add_option("--" + output_prefix_str, type="string",
                         dest="output_prefix",
                         help=("Prefix for output files"))
    argparser.set_defaults(output_prefix=None)

    taglist_file_str = str("taglist")
    argparser.add_option("--" + taglist_file_str, type="string",
                         dest="taglist_name",
                         help=("File containing particle tags"))
    argparser.set_defaults(taglist_name=None)

    # Process the command line list

    opts, args = argparser.parse_args()

    argc = len(args)

    min_args = int(4)

    if (argc < min_args) :
        stderr.write(usage + "\n\n")
        exit(1)

    argptr = int(0)

    filename_prefix = str(args[argptr])
    argptr += 1

    start_index = int(args[argptr])
    argptr += 1

    end_index = int(args[argptr])
    argptr += 1

    data_names = list()

    while (argptr < argc) :
        data_names.append(str(args[argptr]))
        argptr += 1

    output_prefix = opts.output_prefix
    taglist_name = opts.taglist_name

    taglist = None

    if (taglist_name is not None) :
        taglist = read_tag_file(taglist_name)


    worker = ParticleWorker(filename_prefix, start_index, end_index,
                            data_names, output_prefix)

    worker.run_loop(taglist)


if (__name__ == "__main__") :
    main()
