// C++ program file decomp_utils.cpp

/*
  By Nathan C. Hearn
     February 10, 2008

  Problem decomposition utilities.
*/


#include "decomp_utils.hpp"
#include <vector>


int simple_linear_decomp(const unsigned int num_parts, 
			 const unsigned int distrib_size,
			 std::vector<unsigned int> & start_part_indexes,
			 std::vector<unsigned int> & unit_part_counts)
  {
  int ret_val = -1;  // Error state

  if (distrib_size > 0)
    {
    const unsigned int min_parts_per_unit = num_parts / distrib_size;

    const unsigned int remaining_parts 
      = num_parts - (distrib_size * min_parts_per_unit);

    const unsigned int excess_parts_per_unit = min_parts_per_unit + 1;

    start_part_indexes.resize(distrib_size);
    unit_part_counts.resize(distrib_size);

    unsigned int current_part_index = 0;

    for (unsigned int index = 0; index < remaining_parts; index++)
      {
      start_part_indexes[index] = current_part_index;
      unit_part_counts[index] = excess_parts_per_unit;

      current_part_index += excess_parts_per_unit;
      }

    for (unsigned int index = remaining_parts; index < distrib_size; index++)
      {
      start_part_indexes[index] = current_part_index;
      unit_part_counts[index] = min_parts_per_unit;

      current_part_index += min_parts_per_unit;
      }

    ret_val = 0;  // OK
    }

  return ret_val;
  }
