// C++ header file traj_mpi_utils.hpp

/*
  By Nathan C. Hearn
     November 20, 2008

  MPI-related utilities for particle trajectory extraction.
*/


#ifndef TRAJ_MPI_UTILS_HPP
#define TRAJ_MPI_UTILS_HPP

#ifdef USE_MPI


#include <mpi.h>
#include <vector>
#include <string>


extern const int mpi_tag_part_tag;
extern const int mpi_tag_time_count;
extern const int mpi_tag_num_elems;
extern const int mpi_tag_time_array;
extern const int mpi_tag_elem_array_base;


int broadcast_name_list_send(const std::vector<std::string> & elem_name_list,
			     const MPI_Comm mpi_comm=MPI_COMM_WORLD);

int broadcast_name_list_recv(std::vector<std::string> & elem_name_list, 
			     const int root_rank=0,
			     const MPI_Comm mpi_comm=MPI_COMM_WORLD);

int send_particle_data(const int dest_rank, const unsigned int particle_tag,
		       const std::vector<double> & time_list,
		       const std::vector< std::vector<double> > & data_elems,
		       const MPI_Comm mpi_comm=MPI_COMM_WORLD);

int recv_particle_data(const int src_rank, unsigned int & particle_tag,
		       std::vector<double> & time_list,
		       std::vector< std::vector<double> > & data_elems,
		       const MPI_Comm mpi_comm=MPI_COMM_WORLD);


#endif  // USE_MPI

#endif  // TRAJ_MPI_UTILS_HPP
