# materials.py

# By Nathan C. Hearn
#    December 5, 2008
#
# Material properties for raytracer


# Source-opacity coefficients (in cm^2 / g) from C. Kuranz 

## "Along strip" X-ray source

poly_opacity_Ti = 3.01e+01
chbr_opacity_Ti = 1.00e+02
foam_opacity_Ti = 2.48e+01

## "Across strip" X-ray source

poly_opacity_Sc = 3.96e+01
chbr_opacity_Sc = 1.30e+02
foam_opacity_Sc = 3.12e+01

## Other X-ray source

poly_opacity_V = 2.25e+01
chbr_opacity_V = 7.85e+01
foam_opacity_V = 1.63e+01


def get_poly_opacity(source_type) :

    ret_val = None

    if (source_type == "Ti") :
        ret_val = poly_opacity_Ti
    elif (source_type == "Sc") :
        ret_val = poly_opacity_Sc
    elif (source_type == "V") :
        ret_val = poly_opacity_V
    else :
        raise RuntimeError("Unrecognized source type [ " + source_type
                           + " ]")

    return ret_val


def get_chbr_opacity(source_type) :

    ret_val = None

    if (source_type == "Ti") :
        ret_val = chbr_opacity_Ti
    elif (source_type == "Sc") :
        ret_val = chbr_opacity_Sc
    elif (source_type == "V") :
        ret_val = chbr_opacity_V
    else :
        raise RuntimeError("Unrecognized source type [ " + source_type
                           + " ]")

    return ret_val


def get_foam_opacity(source_type) :

    ret_val = None

    if (source_type == "Ti") :
        ret_val = foam_opacity_Ti
    elif (source_type == "Sc") :
        ret_val = foam_opacity_Sc
    elif (source_type == "V") :
        ret_val = foam_opacity_V
    else :
        raise RuntimeError("Unrecognized source type [ " + source_type
                           + " ]")

    return ret_val
