#include <iostream>
#include <vector>
#include "rays.hpp"


const unsigned int dims = 3;


void write_ray(const std::vector<double> & start, 
	       const std::vector<double> & end,
	       const std::vector< std::vector<double> > & samples)
  {
  using std::cout;
  using std::endl;

  for (unsigned int i = 0; i < dims; i++)
    cout << start[i] << " ";

  cout << endl;

  const unsigned int num_samples = samples.size();

  for (unsigned int index = 0; index < num_samples; index++)
    {
    for (unsigned int i = 0; i < dims; i++)
      cout << samples[index][i] << " ";

    cout << endl;
    }

  for (unsigned int i = 0; i < dims; i++)
    cout << end[i] << " ";

  cout << endl;
  }


int main(int argc, char * argv[])
  {
  using std::cout;
  using std::cerr;
  using std::endl;

  std::vector<double> source_pos(dims);

  source_pos[0] = 0.0;
  source_pos[1] = -0.045;
  source_pos[2] = 0.045;

  const double source_rad = 0.005;

  const double cyl_y = 0.045;
  const double cyl_z = 0.045;
  const double cyl_rad = 0.045;

  const double y_min = 0.0;
  const double y_max = 0.090;
  const double z_min = 0.0;
  const double z_max = 0.090;

  Rays rays(source_pos, source_rad, cyl_y, cyl_z, cyl_rad, y_min, y_max,
	    z_min, z_max);

  std::vector<double> end_point(dims);

  end_point[0] = 0.0;
  end_point[1] = 0.135;
  end_point[2] = 0.050;

  const double box_width = 0.010;

  const double spacing = 0.001;
  
  std::vector< std::vector<double> > samples;

  rays.get_samples(end_point, box_width, spacing, samples);

  write_ray(source_pos, end_point, samples);

  return 0;
  }
