// C++ header file layer_finder.hpp

/*
  By Nathan C. Hearn
     January 30, 2008

  Utilities for determining layer boundaries for RT setup (position along axis)
*/


#ifndef LAYER_FINDER_HPP
#define LAYER_FINDER_HPP


#include <string>
#include "quickflash_file_datafile.hpp"
#include "mixing_layer.hpp"


int get_maximal_envelope_range(const QuickFlash::File::DataFile & df,
			       const char dataset_name[],
			       const unsigned int axis,
			       const double min_data_value,
			       const double max_data_value,
			       double & min_pos, double & max_pos);


int get_maximal_envelope_range(const QuickFlash::File::DataFile & df,
			       const std::string & dataset_name,
			       const unsigned int axis,
			       const double min_data_value,
			       const double max_data_value,
			       double & min_pos, double & max_pos);


void get_mixing_layer_meanValue(const QuickFlash::File::DataFile & df,
				const char dataset_name[],
				const unsigned int axis, 
				const double min_coord, const double max_coord,
				const double bin_spacing, 
				const double min_bounds_data_value, 
				const double max_bounds_data_value,
				MixingLayer & mixing_layer,
				const bool use_abs_value=false);


void get_mixing_layer_meanValue(const QuickFlash::File::DataFile & df,
				const std::string & dataset_name,
				const unsigned int axis, 
				const double min_coord, const double max_coord,
				const double bin_spacing, 
				const double min_bounds_data_value, 
				const double max_bounds_data_value,
				MixingLayer & mixing_layer, 
				const bool use_abs_value=false);


void get_mixing_layer_meanValue_peakFract(const QuickFlash::File::DataFile & df,
					  const char dataset_name[],
					  const unsigned int axis, 
					  const double min_coord, 
					  const double max_coord,
					  const double bin_spacing, 
					  const double min_bounds_peak_fract, 
					  const double max_bounds_peak_fract,
					  MixingLayer & mixing_layer,
					  const bool use_abs_value=false);


void get_mixing_layer_meanValue_peakFract(const QuickFlash::File::DataFile & df,
					  const std::string & dataset_name,
					  const unsigned int axis, 
					  const double min_coord, 
					  const double max_coord,
					  const double bin_spacing, 
					  const double min_bounds_peak_fract, 
					  const double max_bounds_peak_fract,
					  MixingLayer & mixing_layer, 
					  const bool use_abs_value=false);


void get_mixing_layer_meanValue_massWeight(const QuickFlash::File::DataFile & df,
					   const char dataset_name[],
					   const unsigned int axis, 
					   const double min_coord, 
					   const double max_coord,
					   const double bin_spacing, 
					   const double min_bounds_data_value, 
					   const double max_bounds_data_value,
					   MixingLayer & mixing_layer, 
					   const bool use_abs_value=false);


void get_mixing_layer_meanValue_massWeight(const QuickFlash::File::DataFile & df,
					   const std::string & dataset_name,
					   const unsigned int axis, 
					   const double min_coord, 
					   const double max_coord,
					   const double bin_spacing, 
					   const double min_bounds_data_value, 
					   const double max_bounds_data_value,
					   MixingLayer & mixing_layer, 
					   const bool use_abs_value=false);


#endif  // LAYER_FINDER_HPP
