// C++ header file blockinfo.hpp

/*
  By Nathan C. Hearn
     November 20, 2006

  SWIG-friendly wrapper for QuickFlash::Block::BlockInfo.
*/


#ifndef BLOCKINFO_HPP
#define BLOCKINFO_HPP


#include "../src_lib/quickflash_block_blockinfo.hpp"

class BlockInfo
  {
  public :

    BlockInfo() : data(), block_center(), block_width() { }

    BlockInfo(const QuickFlash::Block::BlockInfo & source) : 
      data(), block_center(), block_width()
      { reset(source); }

    BlockInfo(const BlockInfo & source) : 
      data(), block_center(), block_width() 
      { reset(source); }

    ~BlockInfo() { }

    void reset()
      {
      data.reset();

      block_center.clear();
      block_width.clear();
      }

    void reset(const QuickFlash::Block::BlockInfo & source)
      { 
      if (&source != &data)
	{
	data = source; 

	source.get_block_center(block_center);
	source.get_block_width(block_width);
	}
      }

    void reset(const BlockInfo & source)
      {
      if (&source != this)
	{
	data = source.data;

	block_center = source.block_center;
	block_width = source.block_width;
	}
      }

    unsigned int get_space_dims() const { return data.get_space_dims(); }

    void get_block_dims(std::vector<unsigned int> & vect) const 
      { vect = data.get_block_dims(); }

    const std::vector<unsigned int> & get_block_dims() const 
      { return data.get_block_dims(); }

    void get_min_coords(std::vector<double> & vect) const
      { vect = data.get_min_coords(); }

    const std::vector<double> & get_min_coords() const 
      { return data.get_min_coords(); }

    void get_max_coords(std::vector<double> & vect) const
      { vect = data.get_max_coords(); }

    const std::vector<double> & get_max_coords() const 
      { return data.get_max_coords(); }

    void get_block_center(std::vector<double> & vect) const
      { vect = block_center; }

    const std::vector<double> & get_block_center() const 
      { return block_center; }

    void get_block_width(std::vector<double> & vect) const
      { vect = block_width; }

    const std::vector<double> & get_block_width() const 
      { return block_width; }

    void get_cell_width(std::vector<double> & vect) const
      { vect = data.get_cell_width(); }

    const std::vector<double> & get_cell_width() const 
      { return data.get_cell_width(); }

    double get_cell_volume() const { return data.get_cell_volume(); }

    double get_cell_volume(const std::vector<unsigned int> & cell_index) const
      { return data.get_cell_volume(cell_index); }

    double get_cell_volume(const unsigned int cell_index) const
      { return data.get_cell_volume(cell_index); }

    unsigned int get_num_cells() const { return data.get_num_cells(); }

    bool in_block(const std::vector<double> & coords) const
      { return data.in_block(coords); }

    bool in_cell(const unsigned int cell_index,
		 const std::vector<double> & coords) const
      { return data.in_cell(cell_index, coords); }

    void get_cell_center(const unsigned int cell_index,
			 std::vector<double> & vect) const
      { data.get_cell_center(cell_index, vect); }

    std::vector<double> get_cell_center(const unsigned int cell_index) const
      {
      std::vector<double> vect;

      get_cell_center(cell_index, vect);

      return vect;
      }

    QuickFlash::Block::BlockInfo & get_ref() { return data; }
    const QuickFlash::Block::BlockInfo & get_ref() const { return data; }

  private :

    QuickFlash::Block::BlockInfo data;

    std::vector<double> block_center;
    std::vector<double> block_width;
  };


#endif  // BLOCKINFO_HPP
