// C++ header file datafile.hpp

/*
  By Nathan C. Hearn
     November 27, 2006

  SWIG-friendly wrapper class for QuickFlash::File::Dataset.
*/


#ifndef DATAFILE_HPP
#define DATAFILE_HPP

#include "../src_lib/quickflash_file_datafile.hpp"
#include "dataset.hpp"
#include "siminfo.hpp"
#include "partdata.hpp"


class Datafile
  {
  public :

    Datafile(const std::string & filename, const unsigned int buffer_size=0,
	     const bool skip_particles=false) :
      datafile()
      { reset(filename, buffer_size, skip_particles); }

    ~Datafile() { }

    void reset(const std::string & filename, const unsigned int buffer_size=0,
	       const bool skip_particles=false)
      { datafile.reset(filename, buffer_size, skip_particles); }

    SimInfo get_sim_info() const
      { return SimInfo(datafile.get_sim_info()); }

    MeshInfo get_mesh_info() const
      { return MeshInfo(datafile.get_mesh_info()); }

    Dataset get_dataset(const std::string & dataset_name,
			const unsigned int buffer_size=0) const
      { return Dataset(datafile.get_dataset(dataset_name, buffer_size)); }

    void report_dataset_stats() const
      { return datafile.report_dataset_stats(); }

    PartData get_part_data() const
      { return PartData(datafile.get_part_data()); }

  private :

    QuickFlash::File::DataFile datafile;
  };


#endif  // DATAFILE_HPP
