// C++ header file dataset.hpp

/*
  By Nathan C. Hearn
     November 27, 2006

  SWIG-friendly wrapper for QuickFlash::File::Dataset.

  TODO: Turn dataset into a pointer variable.
*/


#ifndef DATASET_HPP
#define DATASET_HPP

#include <vector>
#include "../src_lib/quickflash_file_dataset.hpp"
#include "../src_lib/quickflash_block_blockdata.hpp"
#include "blockdata.hpp"


class Dataset
  {
  public :

    Dataset(const QuickFlash::File::Dataset & source) : 
      dataset(0) 
      { reset(source); }

    Dataset(const Dataset & source) : dataset(0) { reset(source); }

    ~Dataset() 
      { 
      dataset = 0;  // Do not deallocate!! 
      }

    void reset(const QuickFlash::File::Dataset & source) { dataset = &source; }

    void reset(const Dataset & source)
      {
      if (&source != this)
	dataset = source.dataset;
      }

    unsigned int get_num_blocks() const { return dataset->get_num_blocks(); }
    unsigned int get_buffer_size() const { return dataset->get_buffer_size(); }
    unsigned int get_block_size() const { return dataset->get_block_size(); }

    double get_min_value() const { return dataset->get_min_value(); }
    double get_max_value() const { return dataset->get_max_value(); }

    void set_cache_size(const unsigned int max_cache_size=0,
			const unsigned int clean_cache_size=0) const
      { dataset->set_cache_size(max_cache_size, clean_cache_size); }

    unsigned int get_cache_size() const { return dataset->get_cache_size(); }

    void set_report_stats(const bool state=true) const
      { dataset->set_report_stats(state); }

    void report_all_stats(const bool report_if_unused=false) const
      { dataset->report_all_stats(report_if_unused); }

    void get_block_data(const unsigned int block_index, 
			QuickFlash::Block::BlockData<double> & data) const
      { dataset->get_block_data(block_index, data); }

    void get_block_data(const unsigned int block_index,
			BlockData<double> & data) const
      { dataset->get_block_data(block_index, data.get_ref()); }

    BlockData<double> get_block_data(const unsigned int block_index) const
      {
      QuickFlash::Block::BlockData<double> data;

      dataset->get_block_data(block_index, data); 

      return data;
      }

  private :

    const QuickFlash::File::Dataset * dataset;
  };


#endif  // DATASET_HPP
