// C++ header file meshinfo.hpp

/*
  By Nathan C. Hearn
     October 6, 2008

  SWIG-friendly wrapper for QuickFlash::File::MeshInfo.
*/


#ifndef MESHINFO_HPP
#define MESHINFO_HPP


#include "../src_lib/quickflash_file_meshinfo.hpp"
#include "blockinfo.hpp"
#include "treeiter.hpp"


class MeshInfo
  {
  public:
    MeshInfo(const MeshInfo & source) : meshinfo(0) { reset(source); }

    MeshInfo(const QuickFlash::File::MeshInfo & source) : 
      meshinfo(0)
      { reset(source); }

    ~MeshInfo()
      { 
      // Do NOT deallocate meshinfo!!
      meshinfo = 0;
      }

    void reset(const MeshInfo & source)
      {
      if (&source != this)
	meshinfo = source.meshinfo;
      }

    void reset(const QuickFlash::File::MeshInfo & source)
      { meshinfo = &source; }

    double get_sim_time() const { return meshinfo->get_sim_time(); }

    unsigned int get_dims() const { return meshinfo->get_dims(); }

    unsigned int get_num_blocks() const { return meshinfo->get_num_blocks(); }

    const std::vector<double> & get_volume_minbounds() const 
      { return meshinfo->get_volume_minbounds(); }

    const std::vector<double> & get_volume_maxbounds() const 
      { return meshinfo->get_volume_maxbounds(); }

    bool is_leaf(const unsigned int block_index) const
      { return meshinfo->is_leaf(block_index); }

    bool is_root(const unsigned int block_index) const
      { return meshinfo->is_root(block_index); }

    void get_block_info(const unsigned int block_index, BlockInfo & blockinfo)
      const
      { blockinfo.reset(meshinfo->get_block_info(block_index)); }

    BlockInfo get_block_info(const unsigned int block_index) const
      { return meshinfo->get_block_info(block_index); }

    void get_block_info(const std::vector<double> & position,
			BlockInfo & blockinfo) const
      { blockinfo.reset(meshinfo->get_block_info(position)); }

    BlockInfo get_block_info(const std::vector<double> & position) const
      { return meshinfo->get_block_info(position); }

    TreeIter get_tree_iter() const
      { return meshinfo->get_tree_iter(); }

  private:
    const QuickFlash::File::MeshInfo * meshinfo;
  };


#endif  // MESHINFO_HPP
