// C++ header file partdata.hpp

/*
  By Nathan C. Hearn
     July 3, 2007

  SWIG-friendly wrapper for QuickFlash::Particles::PartData.
*/


#ifndef PARTDATA_HPP
#define PARTDATA_HPP


#include <vector>
#include <string>
#include "../src_lib/quickflash_particles_data.hpp"


class PartData
  {
  public :

    PartData() : partdata(0) { }

    PartData(const QuickFlash::Particles::PartData & source) :
      partdata(0)
      { reset(source); }

    PartData(const PartData & source) : partdata(0) { reset(source); }

    ~PartData()
      {
      partdata = 0;  // Do NOT deallocate!!!
      }

    void reset() { partdata = 0; }

    void reset(const QuickFlash::Particles::PartData & source) 
      { partdata = &source; }

    void reset(const PartData & source)
      {
      if (&source != this)
	partdata = source.partdata;
      }

    bool particles_active() const { return partdata->particles_active(); }

    unsigned int get_num_particles() const 
      { return partdata->get_num_particles(); }

    bool particle_present(const unsigned int particle_tag) const 
      { return partdata->particle_present(particle_tag); }

    unsigned int get_particle_index(const unsigned int particle_tag) const
      { return partdata->get_particle_index(particle_tag); }

    unsigned int get_particle_tag(const unsigned int particle_index) const
      { return partdata->get_particle_tag(particle_index); }

    unsigned int get_num_variables() const 
      { return partdata->get_num_variables(); }

    bool variable_present(const std::string & var_name) const
      { return partdata->variable_present(var_name); }

    unsigned int get_variable_index(const std::string & var_name) const
      { return partdata->get_variable_index(var_name); }

    const std::string & get_variable_name(const unsigned int var_index) const
      { return partdata->get_variable_name(var_index); }

    std::vector<double> get_variable_data(const unsigned int var_index) const
      {
      std::vector<double> data;

      partdata->get_variable_data(var_index, data);

      return data;
      }

    std::vector<double> get_variable_data(const std::string & var_name) const
      {
      std::vector<double> data;

      partdata->get_variable_data(var_name, data);

      return data;
      }

    std::vector<double> get_particle_data_index(const unsigned int part_index)
      const
      {
      std::vector<double> data;

      partdata->get_particle_data_index(part_index, data);

      return data;
      }
 
    std::vector<double> get_particle_data_tag(const unsigned int part_tag)
      const
      {
      std::vector<double> data;

      partdata->get_particle_data_tag(part_tag, data);

      return data;
      }
 

  private :

    const QuickFlash::Particles::PartData * partdata;
  };


#endif  // PARTDATA_HPP
