// C++ header file siminfo.hpp

/*
  By Nathan C. Hearn
     November 21, 2006

  SWIG-friendly wrapper for QuickFlash::File::SimInfo.
*/


#ifndef SIMINFO_HPP
#define SIMINFO_HPP


#include "../src_lib/quickflash_file_siminfo.hpp"
#include "blockinfo.hpp"
#include "treeiter.hpp"


class SimInfo
  {
  public :

    SimInfo(const SimInfo & source) : siminfo(0) { reset(source); }

    SimInfo(const QuickFlash::File::SimInfo & source) : 
      siminfo(0)
      { reset(source); }

    ~SimInfo()
      { 
      // Do NOT deallocate siminfo!!
      siminfo = 0;
      }

    void reset(const SimInfo & source)
      {
      if (&source != this)
	siminfo = source.siminfo;
      }

    void reset(const QuickFlash::File::SimInfo & source)
      { siminfo = &source; }

    double get_sim_time() const { return siminfo->get_sim_time(); }

    unsigned int get_mesh_dims() const { return siminfo->get_mesh_dims(); }

    unsigned int get_num_blocks() const { return siminfo->get_num_blocks(); }
    unsigned int get_num_particles() const 
      { return siminfo->get_num_particles(); }

    const std::vector<double> & get_volume_minbounds() const 
      { return siminfo->get_volume_minbounds(); }

    const std::vector<double> & get_volume_maxbounds() const 
      { return siminfo->get_volume_maxbounds(); }

  private :

    const QuickFlash::File::SimInfo * siminfo;
  };


#endif  // SIMINFO_HPP
