// C++ header file treeiter.hpp

/*
  By Nathan C. Hearn
     May 3, 2007

  Iterator for tree traversal.
*/


#ifndef TREEITER_HPP
#define TREEITER_HPP


#include <vector>
#include "../src_lib/quickflash_tree_block.hpp"
#include "blockinfo.hpp"


class TreeIter
  {
  public:
    TreeIter() : iterator() { reset(); }

    TreeIter(const QuickFlash::Tree::ConstTreeIter & source) : 
      iterator()
      { reset(source); }

    TreeIter(const TreeIter & source) :
      iterator()
      { reset(source); }

    ~TreeIter() { }

    TreeIter & operator=(const QuickFlash::Tree::ConstTreeIter & source)
      {
      reset(source);
      return *this;
      }

    TreeIter & operator=(const TreeIter & source)
      {
      reset(source);
      return *this;
      }

    void reset() { iterator.reset(); }

    void reset(const QuickFlash::Tree::ConstTreeIter & source)
      { iterator = source; }

    void reset(const TreeIter & source)
      {
      if (&source != this)
	{
	iterator = source.iterator;
	}
      }

    BlockInfo get_block_info() const
      { return iterator.get_block_info(); }

    unsigned int get_block_index() const
      { return iterator.get_block_index(); }

    bool valid_node() const
      { return iterator.valid_node(); }

    bool is_leaf() const
      { return iterator.is_leaf(); }

    int next() const
      { return iterator.next(); }

    int prev() const
      { return iterator.prev(); }

    int next_branch() const
      { return iterator.next_branch(); }

    int first() const
      { return iterator.first(); }

    int last() const
      { return iterator.last(); }

    int get_level() const
      { 
      int ret_val = -1;  // Error state

      if (valid_node())
	{
	const unsigned int node_level
	  = iterator.get_node_iter().get_nodeID().get_level(); 

	ret_val = static_cast<int>(node_level);
	}

      return ret_val;
      }

  private:
    QuickFlash::Tree::ConstTreeIter iterator;
  };


#endif  // TREEITER_HPP
