# grid_fft_ops.py

# By Nathan C. Hearn
#    June 24, 2008
#
# 2D FFT operations


def fft_1d(fft_axis, data) :

    from numeric_defs import numeric_zeros, numeric_float, numeric_cplx, \
         numeric_fft_real

    shape = data.shape

    if (len(shape) != 2) :
        raise RuntimeError("Only 2D data fields supported")

    i_size = shape[0]
    j_size = shape[1]

    array_len = i_size
    num_arrays = j_size

    if (fft_axis == 1) :
        array_len = j_size
        num_arrays = i_size

    work_array = numeric_zeros(array_len, numeric_float)

    trans_array_size = (array_len / 2) + ((array_len + 1) % 2)

    trans_data_size = list(shape)

    trans_data_size[fft_axis] = trans_array_size

    trans_data = numeric_zeros(trans_data_size, numeric_cplx)

    for array_index in range(num_arrays) :

        for elem_index in range(array_len) :

            value = float(0)

            if (fft_axis == 0) :
                value = data[elem_index][array_index]
            else :
                value = data[array_index][elem_index]

            work_array[elem_index] = value

        trans_array = numeric_fft_real(work_array)

        if (len(trans_array) != trans_array_size) :
            raise RuntimeError("Mismatch in transform array sizes")

        for elem_index in range(trans_array_size) :

            value = trans_array[elem_index]

            if (fft_axis == 0) :
                trans_data[elem_index][array_index] = value
            else :
                trans_data[array_index][elem_index] = value

    return trans_data


def split_real_imag_arrays(data) :

    from numeric_defs import numeric_zeros, numeric_float

    shape = data.shape

    if (len(shape) != 2) :
        raise RuntimeError("Only 2D data fields supported")

    i_size = shape[0]
    j_size = shape[1]

    real_data = numeric_zeros(shape, numeric_float)
    imag_data = numeric_zeros(shape, numeric_float)

    for i in range(i_size) :
        for j in range(j_size) :

            value_cplx = data[i][j]

            real_data[i][j] = value_cplx.real
            imag_data[i][j] = value_cplx.imag

    return real_data, imag_data


def fft_1d_power(axis, trans_data) :

    from numeric_defs import numeric_zeros, numeric_float

    shape = trans_data.shape

    if (len(shape) != 2) :
        raise RuntimeError("Only 2D data fields supported")

    i_size = shape[0]
    j_size = shape[1]

    power_data = numeric_zeros(shape, numeric_float)

    for i in range(i_size) :
        for j in range(j_size) :

            cplx_value = trans_data[i][j]

            ## DO WE NEED TO TREAT THE FIRST AND LAST ELEMENTS DIFFERENTLY???

            re = cplx_value.real
            im = cplx_value.imag

            power = 2.0 * ((re * re) + (im * im))

            power_data[i][j] = power

    return power_data
