#!/usr/bin/env python

# plot_demo.py

# By Nathan C. Hearn
#    July 29, 2008
#
# Demonstration program showing capabilities of generate_plot.py


def main() :

    from sys import exit, stderr
    from os import path
    from optparse import OptionParser

    from read_table import read_table
    from generate_plot import Plot


    ## Set up the command line parser

    usage = str("%prog [options] data_file "
                + "indep_elem1,dep_elem1[,elem1_title] [ indep_elem2,... ]")
    version = str("%prog 0.1")

    argparser = OptionParser(usage, version=version)

    output_file_str = str("output-file")
    argparser.add_option("--" + output_file_str, type="string",
                         dest="output_file", help="Output image filename")
    argparser.set_defaults(output_file=None)

    plot_title_str = str("plot-title")
    argparser.add_option("--" + plot_title_str, type="string",
                         dest="plot_title", help="Plot title")
    argparser.set_defaults(plot_title=None)

    output_format_str = str("output-format")
    argparser.add_option("--" + output_format_str, type="string",
                         dest="output_format",
                         help="Name of matplotlib backend")
    argparser.set_defaults(output_format=None)

    x_axis_title_str = str("x-axis-title")
    argparser.add_option("--" + x_axis_title_str, type="string",
                         dest="x_axis_title", help="Title for x-axis")
    argparser.set_defaults(x_axis_title=None)

    y_axis_title_str = str("y-axis-title")
    argparser.add_option("--" + y_axis_title_str, type="string",
                         dest="y_axis_title", help="Title for y-axis")
    argparser.set_defaults(y_axis_title=None)

    x_logscale_str = str("x-logscale")
    argparser.add_option("--" + x_logscale_str, action="store_true",
                         dest="x_logscale", help="Use logscale for x-axis")
    argparser.set_defaults(x_logscale=False)

    y_logscale_str = str("y-logscale")
    argparser.add_option("--" + y_logscale_str, action="store_true",
                         dest="y_logscale", help="Use logscale for x-axis")
    argparser.set_defaults(y_logscale=False)

    ## Process the command line input

    opts, args = argparser.parse_args()

    argc = len(args)

    min_args = int(2)

    if (argc < min_args) :
        stderr.write("\n")
        argparser.print_help(file=stderr)
        stderr.write("\n")
        exit(1)

    ## Get the required arguments

    argptr = int(0)

    data_file = str(args[argptr])
    argptr += 1

    indep_var_list = list()
    dep_var_list = list()
    dep_var_title_list = list()

    while (argptr < argc) :
        
        current_elem_str = args[argptr]
        argptr += 1

        elem_words = current_elem_str.split(",")

        num_words = len(elem_words)

        if ((num_words < 2) or (num_words > 3)) :
            stderr.write("\nError: Improper format for plot variable "
                         + "specification\n\n")
            exit(-1)

        indep_var = elem_words[0]
        dep_var = elem_words[1]

        dep_var_title = dep_var

        if (num_words > 2) :
            dep_var_title = elem_words[2]

        indep_var_list.append(indep_var)
        dep_var_list.append(dep_var)
        dep_var_title_list.append(dep_var_title)

    num_sets = len(indep_var_list)

    ## Process the options

    output_file = opts.output_file  ## If None, will output to screen

    plot_title = opts.plot_title

    if (plot_title is None) :
        plot_title = data_file

    output_format = opts.output_format

    if ((output_format is None) and (output_file is None)) :
        output_format = "wxAgg"  ## More general backend??

    indep_axis_title = opts.x_axis_title

    if (indep_axis_title is None) :
        indep_axis_title = indep_var_list[0]

    dep_axis_title = opts.y_axis_title

    if (dep_axis_title is None) :
        dep_axis_title = dep_var_list[0]

    x_logscale = opts.x_logscale
    y_logscale = opts.y_logscale

    ## Read the data

    plot_data = read_table(data_file)

    ## Build the plot

    plot = Plot(plot_title, indep_axis_title, dep_axis_title)

    plot.set_logscale_x(x_logscale)
    plot.set_logscale_y(y_logscale)

    for set_index in range(num_sets) :

        indep_var = indep_var_list[set_index]
        dep_var = dep_var_list[set_index]
        
        dep_var_title = dep_var_title_list[set_index]

        indep_data = plot_data[indep_var]

        dep_data = plot_data[dep_var]

        plot.add_plot(indep_data, dep_data, legend_text=dep_var_title)

    plot.show_plot(output_file, output_format)


if (__name__ == "__main__") :
    main()
