# read_table.py

# By Nathan C. Hearn
#    July 29, 2008
#
# Loading of tables from file


def read_line(file_object, ignore_leading_comment_mark=False) :

    ret_val = None

    reached_eof = False
    found_line = False

    while ((not reached_eof) and (not found_line)) :

        line = file_object.readline()

        if (line == "") :
            
            reached_eof = True
            
        else :

            line = line.strip()

            if (len(line) > 0) :

                if (ignore_leading_comment_mark) :
                    if (line[0] == "#") :
                        line = line[1:]

                no_comment_line = line.split("#")[0].strip()

                if (no_comment_line != "") :

                    ret_val = no_comment_line

                    found_line = True

    return ret_val


def read_table(file_object, table_start_line=None,
               ignore_leading_comment_mark=False) :

    f = None

    if (isinstance(file_object, str)) :
        f = open(file_object, "r")
    elif (isinstance(file_object, file)) :
        f = file_object
    else :
        raise RuntimeError("Unrecognized file object")

    end_reached = False

    # Advance to the table start if desired

    if (table_start_line is not None) :

        start_found = False

        while ((not start_found) and (not end_reached)) :

            current_line = read_line(f, ignore_leading_comment_mark)

            if (current_line is None) :
                
                end_reached = True

            elif (current_line.strip() == table_start_line) :

                start_found = True

        if (end_reached) :
            raise RuntimeError("End of file reached before table found")

    # Read the headings

    heading_line = read_line(f)

    if (heading_line is None) :
        raise RuntimeError("Unable to find headings in file " + filename)

    headings = heading_line.split()

    num_columns = len(headings)

    data = list()

    for i in range(num_columns) :
        data.append(list())

    # Load the data

    end_reached = False

    while (not end_reached) :

        data_line = read_line(f)

        if (data_line is None) :

            end_reached = True

        else :

            data_elements = data_line.split()

            if (len(data_elements) != num_columns) :
                raise RuntimeError("Column-count mismatch")

            for i in range(num_columns) :
                data[i].append(float(data_elements[i]))

    # Construct the table

    table = dict()

    for i in range(num_columns) :

        column = data[i]

        column_heading = headings[i]

        if (table.has_key(column_heading)) :
            raise RuntimeError("Duplicate column heading")

        table[column_heading] = column

    return table

    
def read_keywords_table(file_object, table_start_line="Data",
                        ignore_leading_comment_mark=False) :

    from keyword_values import KeywordValueList

    f = None

    if (isinstance(file_object, str)) :
        f = open(file_object, "r")
    elif (isinstance(file_object, file)) :
        f = file_object
    else :
        raise RuntimeError("Unrecognized file object")

    eof_found = False

    ## Read the keywords

    keyword_value_list = KeywordValueList()

    data_reached = False

    while ((not data_reached) and (not eof_found)) :

        keyword_line = read_line(f, ignore_leading_comment_mark)

        if (keyword_line is None) :

            eof_found = True

        elif (keyword_line == table_start_line) :

            data_reached = True

        else :

            keyword_split_words = keyword_line.split()

            num_words = len(keyword_split_words)

            if (num_words > 0) :

                keyword = keyword_split_words[0]

                value_string = None

                if (num_words > 1) :
                    
                    ## Remove the keyword for keyword_line

                    keyword_len = len(keyword)

                    value_string = keyword_line[keyword_len:].strip()

                keyword_value_list.add_keyword_value(keyword, value_string)

    ## Read the table

    table = read_table(f)

    ## Return the keyword-value pairs and the table

    return keyword_value_list, table
