// C++ header file random_value.hpp

/*
  By Nathan C. Hearn
     June 14, 2008

  Class to produce uniform random numbers from a given range.
*/


#ifndef RANDOM_VALUE_HPP
#define RANDOM_VALUE_HPP


#include <cstdlib>
#include "quickflash_except.hpp"


class RandomValue
  {
  public :

    RandomValue(const double min_value=0.0, const double max_value=1.0) :
      scale_from_int(0.0), offset(0.0)
      { reset(min_value, max_value); }

    RandomValue(const RandomValue & source) :
      scale_from_int(0.0), offset(0.0)
      { reset(source); }

    ~RandomValue() { }

    RandomValue & operator=(const RandomValue & source)
      {
      reset(source);
      return *this;
      }

    void reset(const double min_value=0.0, const double max_value=1.0)
      {
      if (min_value > max_value)
	throw QuickFlash::Except("Min value can not exceed max value", 
				 __FILE__, __LINE__);

      const double delta_value = max_value - min_value;

      scale_from_int = delta_value / static_cast<double>(RAND_MAX);

      offset = min_value;
      }

    void reset(const RandomValue & source)
      {
      if (&source != this)
	{
	scale_from_int = source.scale_from_int;
	offset = source.offset;
	}
      }

    double get_value() const
      {
      double range_value = 0.0;

      if (scale_from_int > 0.0)
	range_value = static_cast<double>(std::rand()) * scale_from_int;

      return range_value + offset;
      }

  private :

    double scale_from_int;

    double offset;
  };


#endif  // RANDOM_VALUE_HPP
