// C++ header file star_array.hpp

/*
  By Nathan C. Hearn
     June 14, 2008

  Set of stars and routines to compute composite color.
*/


#include "star_array.hpp"
#include <list>
#include "quickflash_color_color.hpp"


// Class StarArray


StarArray::StarArray() : star_list() { }


StarArray::StarArray(const std::list<Star> & source_star_list) : star_list()
  { reset(source_star_list); }


StarArray::StarArray(const StarArray & source) : star_list() { reset(source); }


void StarArray::reset()
  {
  star_list.clear();
  }


void StarArray::reset(const std::list<Star> & source_star_list)
  {
  star_list = source_star_list;
  }


void StarArray::reset(const StarArray & source)
  {
  if (&source != this)
    {
    star_list = source.star_list;
    }
  }


void StarArray::get_color(const double x, const double y, 
			  QuickFlash::Color::Color & color) const
  {
  // Reset the color to transparent black

  color.reset(0.0, 0.0);

  // Run through the list of stars, adding the colors

  QuickFlash::Color::Color work_color;

  std::list<Star>::const_iterator star_iter = star_list.begin();

  while (star_iter != star_list.end())
    {
    const Star & current_star = *star_iter;

    if (current_star.get_color(x, y, work_color))
      color.overlay_color(work_color);

    ++star_iter;
    }
  }
